% MSH_slip_dil_tend.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     5/26/2017
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:  Calculate slip and dilation tendency using stress derived from
% geodetic strainrates and new fault lineaments from lidar, geophysics, and
% geologic mapping gathered during phase 2 of the Washington State Play
% Fairway Analysis in 2D at 200m and 2km depths.
%
% PURPOSE OF MODIFICATION:
%
% INPUT: Comma delimited files with fault verticies in cartesian
% coordinates (not lat/lon) numbered along fault trace and with unique
% number identifier for each set of points along each individual fault
% exported from ArcGIS and using QGIS:
%   -MSH_200m.csv
%   -MSH_2km.csv
% Stress tensor with orientation relative to cartesian coordinate
% system used for fault verticies:
%   -StressTensorMSH200m
%   -StressTensorMSH2km
%
% OUTPUT: slip and dilation tendency calculated using Cauchies Law between
% fault verticies for each individual fault (ST_DT_200m and ST_DT_2km). The
% first fault vertex is assigned NaN and slip/dilation tendencies are
% assigned to the 2nd point of each fault segment between two verticies.
%
% NOTE: This script uses the same 3D stress tensor used in the Poly3D
% modeling for th Washington State Play Fairway Analysis, but can be
% modified to use a 2D stress tensor by commenting out the vertical
% component of the unit normal vector for each fault segment (lines 83 and
% 107) which in this case is set to zero. Likewise, this script can be used
% for 3D slip tendency by adding a vertical component to the unit normal
% vector for each fault segment with the condition that the 3 combined
% components of the unit normal vector still have a magnitude of 1.
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%

clear all;close all;

%% import fault points
XYpoints200m = importdata('MSH_200m.csv',',');
XYpoints2km = importdata('MSH_2km.csv',',');

%% Seprate nodes by ID_SHAPE attribute (individual polylines with ordered points along each line)
% 200m
N200 = unique(XYpoints200m.data(:,1));
for i = 1:numel(N200)
    I = find(XYpoints200m.data(:,1)==N200(i));
    faults200{i,1} = XYpoints200m.data(I,:);
end

% 2km
N2km = unique(XYpoints2km.data(:,1));
for i = 1:numel(N2km)
    I = find(XYpoints2km.data(:,1)==N2km(i));
    faults2km{i,1} = XYpoints2km.data(I,:);
end

%% import stress tensors
Sigma200 = importdata('StressTensorMSH200m.csv',',');
Sigma2km = importdata('StressTensorMSH2km.csv',',');

%% principal stresses from eigenvalues
[V200,PS200] = eig(Sigma200);
[V2km,PS2km] = eig(Sigma2km);

%% Normal Vectors --> Slip/Dilation Tendencies
% 200m
for j=1:numel(faults200)
    for i=2:numel(faults200{j}(:,1))
        x1 = faults200{j}(i-1,4);x2 = faults200{j}(i,4);% line segment x-points
        y1 = faults200{j}(i-1,5);y2 = faults200{j}(i,5);% line segment y-points
        l = ((x2-x1)^2+(y2-y1)^2)^.5;% length of line segment
        nvec(1,1) = (y2-y1)/l; nvec(2,1) = -(x2-x1)/l;nvec(3,1) = 0;% unit normal vector
        T = Sigma200*nvec;% Tractions (Cauchie's Law)
        Tn = -T'*nvec;% Normal Tractions
        Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
        DC = Tn/Tmag;% Direction Cosines
        theta = acos(DC);% Traction angle from normal vector
        Ts = Tmag*sin(theta);% Shear Tractions
        ST_200m(i-1,1) = Ts/Tn;% Slip Tendencies
        DT_200m(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    end
    faults200{j}(1,6:7) = NaN;
    faults200{j}(2:numel(faults200{j}(:,1)),6) = ST_200m;
    faults200{j}(2:numel(faults200{j}(:,1)),7) = DT_200m;
    clear x1;clear x2;clear y1;clear y2;clear ST_200m;clear DT_200m;
end

% 2km
for j=1:numel(faults2km)
    for i=2:numel(faults2km{j}(:,1))
        x1 = faults2km{j}(i-1,4);x2 = faults2km{j}(i,4);% line segment x-points
        y1 = faults2km{j}(i-1,5);y2 = faults2km{j}(i,5);% line segment y-points
        l = ((x2-x1)^2+(y2-y1)^2)^.5;% length of line segment
        nvec(1,1) = (y2-y1)/l;nvec(2,1) = -(x2-x1)/l;nvec(3,1) = 0;% unit normal vector
        T = Sigma2km*nvec;% Tractions (Cauchie's Law)
        Tn = -T'*nvec;% Normal Traction
        Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
        DC = Tn/Tmag;% Direction Cosines
        theta = acos(DC);% Traction angle from normal vector
        Ts = Tmag*sin(theta);% Shear Traction
        ST_2km(i-1,1) = Ts/Tn;% Slip Tendencies
        DT_2km(i-1,1) = ((-PS2km(1,1))-(Tn))/((-PS2km(1,1))-(-PS2km(3,3)));% Dilation Tendencies
    end
    faults2km{j}(1,6:7) = NaN;
    faults2km{j}(2:numel(faults2km{j}(:,1)),6) = ST_2km;
    faults2km{j}(2:numel(faults2km{j}(:,1)),7) = DT_2km;
    clear x1;clear x2;clear y1;clear y2;clear ST_2km;clear DT_2km;
end
%% Column data for slip and dilation tendency
% Columns: 1)fault identifier, 2)blank column (all zeros, artifact from
% GIS), 3)fault vertex number, 4)vertex easting 5)vertex northing 6)slip
% tendency, and 7)dilation tendency. Column headers are defined on last
% line of this script.

% 200m
for i=1:numel(faults200)
    if i==1
        ST_DT_200m = faults200{i};
    else
        ST_DT_200m = [ST_DT_200m;faults200{i}];
    end
end

% 200m
for i=1:numel(faults2km)
    if i==1
        ST_DT_2km = faults2km{i};
    else
        ST_DT_2km = [ST_DT_2km;faults2km{i}];
    end
end

colheaders = {XYpoints200m.textdata{1,3:7} 'slip tendency' 'dilation tendency'};