% WRV_Plot_3D_North_Crossfault.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/08/28
% DATE MODIFIED:    2017/05/16
% VERSION:          v.3
%
% PURPOSE:  To construct 3D fault geometry of North Crossfault
%
% PURPOSE OF MODIFICATION:  Modified from
% 'WRV_Plot_3D_BunkerHill_refined.m' to model ENE-WSW crossfault at Mount
% St Helens
% 
% INPUT: Surface points from mapped fault trace:
%       North_Crossfault_surfacepoints.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-),
% triangulation of fault nodes (tn-), and unit normal vectors of fault element
% triangles (nvec-).
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       fitNormal.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 1;% Show graph of fit normal (0=no 1=yes)
f = 1;% smoothing factor (0 to 1)
convention = 'z-';% PlotTriNormals vector direction
%% bunker hill fault %%

% local origin
xo = 5.56e5;
yo = 5.129e6;

% import data
data = importdata('North_Crossfault_surfacepoints.csv',',');
x = data.data(:,1) - xo;
y = data.data(:,2) - yo;
z = -ones(numel(x),1);

nsp = numel(x);

% fit normal vector
if show_graph==1
    figure
    clear c;clear n
    c(:,1) = x;c(:,2) = y;c(:,3) = z;
    n = fitNormal(c, show_graph);
end

% rotate to j=0
theta = atan((max(x)-min(x))/(max(y)-min(y)));
Rccw = [cos(theta) -sin(theta);sin(theta) cos(theta)];
dx = 500;
dy = 0;
for i=1:numel(x)
    b = Rccw*[x(i);y(i)];
    x(i) = b(1)+dx;y(i) = b(2)+dy;
end

% fit normal vector
if show_graph==1
    figure
    clear c;clear n
    c(:,1) = x;c(:,2) = y;c(:,3) = z;
    n = fitNormal(c, show_graph);
end

% project surface trace to depth
for i=1:22
    x(numel(x)+1:numel(x)+nsp) = x(1:nsp);
    y(numel(y)+1:numel(y)+nsp) = y(1:nsp);
    z(numel(z)+1:numel(z)+nsp) = -i*150;
end

% fit normal vector
if show_graph==1
    figure
    clear c;clear n
    c(:,1) = x;c(:,2) = y;c(:,3) = z;
    n = fitNormal(c, show_graph);
end

% find fault edges
q = unique(z);
for i=1:numel(q)
    I = find(z==q(i));
    J = find(y(I)==min(y(I)));
    xs(i) = x(I(J));ys(i) = y(I(J));zs(i) = z(I(J));
    J = find(y(I)==max(y(I)));
    xn(i) = x(I(J));yn(i) = y(I(J));zn(i) = z(I(J));
end
% fit normal vector
if show_graph==1
    figure
    clear c;clear n
    c(:,1) = [xs';xn'];c(:,2) = [ys';yn'];c(:,3) = [zs';zn'];
    n = fitNormal(c, show_graph);
end

% I = find(ys<400);
% d = xs(I);
% clear xs;
% xs = d;
% clear d
% d = ys(I);
% clear ys;
% ys = d;
% clear d
% d = zs(I);
% clear zs;
% zs = d;
% clear d
% 
% I = find(yn>3400);
% d = xn(I);
% clear xn;
% xn = d;
% clear d
% d = yn(I);
% clear yn;
% yn = d;
% clear d
% d = zn(I);
% clear zn;
% zn = d;
% clear d

% add curvilinear fault tips
for i=1:12
    x = [x;xs(round(2+.3*i^2):numel(xs)-round(1+.3*i^2))'];
    y = [y;ys(round(2+.3*i^2):numel(ys)-round(1+.3*i^2))'-150*i];
    z = [z;zs(round(2+.3*i^2):numel(zs)-round(1+.3*i^2))'];
    x = [x;xn(round(2+.3*i^2):numel(xn)-round(1+.3*i^2))'];
    y = [y;yn(round(2+.3*i^2):numel(yn)-round(1+.3*i^2))'+150*i];
    z = [z;zs(round(2+.3*i^2):numel(zs)-round(1+.3*i^2))'];
end

% x = [x;xn(2:4)'];
% y = [y;yn(2:4)'+800];
% z = [z;zn(2:4)'];

% fit normal vector
if show_graph==1
    figure
    clear c;clear n
    c(:,1) = x;c(:,2) = y;c(:,3) = z;
    n = fitNormal(c, show_graph);
end

% rotate about y-axis
xsu = max(x);
x = x - xsu;
Rccw = [cos(20*(pi/180)) -sin(20*(pi/180));sin(20*(pi/180)) cos(20*(pi/180))];
for i=1:numel(x)
    b = Rccw*[x(i);z(i)];
    x(i) = b(1);z(i) = b(2);
end
x = x + xsu;

% put xyz points for fault nodes in a 3 column matrix
XYZ(:,1) = x;XYZ(:,2) = y;XYZ(:,3) = z;
for i=1:numel(XYZ(:,3))
    if XYZ(i,3)==0
        XYZ(i,3)=-1;
    end
end

% triangulate fault nodes
CON=Delaunay2_5D(XYZ);

% Rotate back
Rcw = [cos(theta) sin(theta);-sin(theta) cos(theta)];
for i=1:numel(XYZ(:,1))
    b = Rcw*[XYZ(i,1)-dx;XYZ(i,2)-dy];
    XYZ(i,1) = b(1);XYZ(i,2) = b(2);
end

% plot fault nodes
figure
plot3(XYZ(:,1),XYZ(:,2),XYZ(:,3),'ko');
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
axis equal;grid on
title('North Crossfault','fontsize',12)
hold on

% plot fault mesh
for i=1:numel(CON(:,1))
    plot3(XYZ([CON(i,:) CON(i,1)],1),XYZ([CON(i,:) CON(i,1)],2),XYZ([CON(i,:) CON(i,1)],3),'b-')
end

% correct polarity of triangulation
[tn,nvec] = PlotTriNormals(CON,XYZ,convention);

% Translate back to UTM
XYZ(:,1) = XYZ(:,1) + xo;
XYZ(:,2) = XYZ(:,2) + yo;