%% MSH_Plot_EQ_3D_northfault.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    2017/05/12
% VERSION:          v.2
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: modified from 'MSH_Plot_EQ_3D_allfaults.m' to
% refine fault mesh.
% 
% INPUT: Seismic catalogs selected from Pacific Northwest Seismic Network:
%           1) pnsn_relocations_faults_north.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-),
% triangulation of fault nodes (tn-), and unit normal vectors of fault element
% triangles (nvec-).
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Import Data

%%% north fault %%%
data = importdata('pnsn_relocations_faults_north.csv',',');
for i=2:numel(data.textdata(:,1))
    datetime = strsplit(data.textdata{i,3},' ');
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{1}) str2num(date{2}) str2num(date{3})];
    time = strsplit(datetime{2},':');
    Time = [str2num(time{1}) str2num(time{2}) str2num(time{3})];
    dnum(i-1,1) = datenum(Date(1),Date(2),Date(3),Time(1),Time(2),Time(3));
    lat(i-1,1) = str2num(data.textdata{i,7});
    long(i-1,1) = str2num(data.textdata{i,8});
    z(i-1,1) = (-str2num(data.textdata{i,5})*1000)-(rand*(10^-7));
    m(i-1,1) = str2num(data.textdata{i,6});
end
S = 3.^m;
[x,y,utmzone] = deg2utm(lat,long);
% UTM origin
xo = 555500;
yo = 5123500;
x = x - xo;
y = y - yo;


%% filter data
% by time
% north fault
J = find(dnum>datenum(1980,1,1)&dnum<datenum(1990,1,1));% 1980s
K = find(dnum>datenum(1990,1,1));% 1990s & 2000s

%% Plot 3D EQs
% north fault
figure
h1 = scatter3(x(J),y(J),z(J),S(J),'k');
hold on
h2 = scatter3(x(K),y(K),z(K),S(K),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 8000])
ylim([0 18000])
zlim([-14000 0])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title('North Fault','fontsize',14)

%% cross sections

% remove events below 14km depth
Q = find(z>-14000);
x = x(Q);y = y(Q);z = z(Q);

% depth slices for node locations
depths = [-13900:300:-1000];depths = depths';
depths2 = [-13900:300:-3400 -3100:150:-1600];depths2 = depths2';
% northing coordinates for cross-sections/node locations
% norths = [900:300:13800];% 300m spacing
norths = [900:300:6600 6750:150:13800];

q = 2;% moving window, # of cells on each side

for i=1:numel(norths)
    if norths(i)<4500
        I = find(y>(norths(i)-3500)&y<=(norths(i)+2500));
        J = find(dnum(I)>datenum(1980,1,1)&dnum(I)<datenum(1990,1,1));% 1980s
        K = find(dnum(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(x(I(J)),z(I(J)),S(I(J)),'k');
        hold on
        scatter(x(I(K)),z(I(K)),S(I(K)),'r');
        axis equal;grid on;
        xlim([0 8000])
        ylim([-14000 0])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
        X{i}.pre1990 = x(I(J));X{i}.post1990 = x(I(K));X{i}.north = norths(i);
        Y{i}.pre1990 = y(I(J));Y{i}.post1990 = y(I(K));Y{i}.north = norths(i);
        Z{i}.pre1990 = z(I(J));Z{i}.post1990 = z(I(K));Z{i}.north = norths(i);
    else
        if norths(i)<7950
            q=4;
            I = find(y>(norths(i-q))&y<=(norths(i+q)));
            J = find(dnum(I)>datenum(1980,1,1)&dnum(I)<datenum(1990,1,1));% 1980s
            K = find(dnum(I)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(I(J)),z(I(J)),S(I(J)),'k');
            hold on
            scatter(x(I(K)),z(I(K)),S(I(K)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990 = x(I(J));X{i}.post1990 = x(I(K));X{i}.north = norths(i);
            Y{i}.pre1990 = y(I(J));Y{i}.post1990 = y(I(K));Y{i}.north = norths(i);
            Z{i}.pre1990 = z(I(J));Z{i}.post1990 = z(I(K));Z{i}.north = norths(i);
        elseif norths(i)>=7950&&norths(i)<=11100
            q=3;
            Is = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z>-6000);% shallow fault
            Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-6000);% deep fault
            Js = find(dnum(Is)>datenum(1980,1,1)&dnum(Is)<datenum(1990,1,1));% 1980s
            Jd = find(dnum(Id)>datenum(1980,1,1)&dnum(Id)<datenum(1990,1,1));% 1980s
            Ks = find(dnum(Is)>datenum(1990,1,1));% 1990s & 2000s
            Kd = find(dnum(Id)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(Is(Js)),z(Is(Js)),S(Is(Js)),'k');
            hold on
            scatter(x(Id(Jd)),z(Id(Jd)),S(Id(Jd)),'k');
            scatter(x(Is(Ks)),z(Is(Ks)),S(Is(Ks)),'r');
            scatter(x(Id(Kd)),z(Id(Kd)),S(Id(Kd)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990s = x(Is(Js));X{i}.post1990s = x(Is(Ks));X{i}.north = norths(i);
            X{i}.pre1990d = x(Id(Jd));X{i}.post1990d = x(Id(Kd));
            Y{i}.pre1990s = y(Is(Js));Y{i}.post1990s = y(Is(Ks));Y{i}.north = norths(i);
            Y{i}.pre1990d = y(Id(Jd));Y{i}.post1990d = y(Id(Kd));
            Z{i}.pre1990s = z(Is(Js));Z{i}.post1990s = z(Is(Ks));Z{i}.north = norths(i);
            Z{i}.pre1990d = z(Id(Jd));Z{i}.post1990d = z(Id(Kd));
        else
            q=3;
            Is = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z>-7000);% shallow fault
            if norths(i)<12600
                Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-7000);% deep fault
            else
                Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-7000&z>-12000);% deep fault
            end
            Js = find(dnum(Is)>datenum(1980,1,1)&dnum(Is)<datenum(1990,1,1));% 1980s
            Jd = find(dnum(Id)>datenum(1980,1,1)&dnum(Id)<datenum(1990,1,1));% 1980s
            Ks = find(dnum(Is)>datenum(1990,1,1));% 1990s & 2000s
            Kd = find(dnum(Id)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(Is(Js)),z(Is(Js)),S(Is(Js)),'k');
            hold on
            scatter(x(Id(Jd)),z(Id(Jd)),S(Id(Jd)),'k');
            scatter(x(Is(Ks)),z(Is(Ks)),S(Is(Ks)),'r');
            scatter(x(Id(Kd)),z(Id(Kd)),S(Id(Kd)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990d = x(Id(Jd));X{i}.post1990d = x(Id(Kd));X{i}.north = norths(i);
            Y{i}.pre1990d = y(Id(Jd));Y{i}.post1990d = y(Id(Kd));Y{i}.north = norths(i);
            Z{i}.pre1990d = z(Id(Jd));Z{i}.post1990d = z(Id(Kd));Z{i}.north = norths(i);
        end
    end
end
    
%% Regression on cross sections
f = 1;% smoothing factor (0 to 1)
wantplot = 0;% zero = no plot
imagefile = 'lowess_plot\';% directory where plot is stored

Xc = [];Yc = [];Zc = [];% nodes south of 8100m north (bifercation point)
Xs = [];Ys = [];Zs = [];% nodes of shallow fault
Xd = [];Yd = [];Zd = [];% nodes of deep fault

for i=1:numel(X)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
    
    if X{1,i}.north>6600
        depths=depths2;
    end

    if X{1,i}.north<7950
        % pre 1990 events
        datain(:,1) = Z{1,i}.pre1990;
        datain(:,2) = X{1,i}.pre1990;
        xdata = depths;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);

        % vertical extent of fault 
        I = find(xdata>=min(Z{1,i}.pre1990)&xdata<=max(Z{1,i}.pre1990));
        
        % plot regression on cross section
        figure(i+1)
        plot(xy(I,2),xy(I,1),'k-');
                
        clear datain
        % post 1990 events
        datain(:,1) = [xy(I,1);Z{1,i}.post1990];
        datain(:,2) = [xy(I,2);X{1,i}.post1990];
        clear xy

        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
        
        % vertical extent of fault
        if X{1,i}.north==900
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
        elseif X{1,i}.north==1200
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata==-11800);
            I = [K;I];
        elseif X{1,i}.north==1500
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata<=-11800&xdata>=-12100);
            I = [K;I];
        elseif X{1,i}.north==1800
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata<=-11800&xdata>=-12400);
            I = [K;I];
        elseif X{1,i}.north==2100
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata<=-11800&xdata>=-12700);
            I = [K;I];
        elseif X{1,i}.north==2400
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-12700);
            I = [K;I];
        elseif X{1,i}.north==2700
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
        elseif X{1,i}.north==3000
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
        elseif X{1,i}.north==4500
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-3100&xdata<=-1600);
            I = [I;J];
        elseif X{1,i}.north==4800
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1900&xdata<=-1600);
            I = [I;J];
        elseif X{1,i}.north==5100
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1900&xdata<=-1600);
            I = [I;J];
        elseif X{1,i}.north==6900
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata==-1300);
            I = [I;J];
        elseif X{1,i}.north==7050
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1300&xdata<=-1150);
            I = [I;J];
        elseif X{1,i}.north==7500
            I = find(xdata>=-13000&xdata<=max(datain(:,1)));
        elseif X{1,i}.north==7800
            I = find(xdata>=-13000&xdata<=max(datain(:,1)));
        else
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
        end
        
        if mod(X{1,i}.north/150,2)~=0
            J = find(xdata(I)>=-3200);
            I = I(J);
        else
        end
        
        % plot regression on cross section
        plot(xy(I,2),xy(I,1),'k--');
        
        % store xyz points for fault nodes
        Xc(numel(Xc)+1:numel(Xc)+numel(I),1) = xy(I,2);
        Yc(numel(Yc)+1:numel(Yc)+numel(I),1) = Y{i}.north;
        Zc(numel(Zc)+1:numel(Zc)+numel(I),1) = xy(I,1);
    else
        % bifercation point
            if X{1,i}.north>=7950&&X{1,i}.north<=11100
            % pre 1990 events
            datains(:,1) = Z{1,i}.pre1990s;
            datains(:,2) = X{1,i}.pre1990s;
            dataind(:,1) = Z{1,i}.pre1990d;
            dataind(:,2) = X{1,i}.pre1990d;
            xdata = depths;
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xys] = lowess(datains,f,wantplot,imagefile,xdata);
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % vertical extent of fault
            if X{1,i}.north==9150
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata>=-2050&xdata<=-1450);
                Is = [Is;Js];
            elseif X{1,i}.north>=9300&&X{1,i}.north<9750
                if X{1,i}.north==9300
                    Is = find(xdata>-5800&xdata<=-1600);
                else
                    Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-1600);
                end
            elseif X{1,i}.north>=9750&&X{1,i}.north<10200
                if X{1,i}.north==9900
                    Is = find(xdata>-5200&xdata<=-1750);
                else
                    Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-1750);
                end
            elseif X{1,i}.north>=10200&&X{1,i}.north<10500
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-1900);
            elseif X{1,i}.north>=10500&&X{1,i}.north<10800
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-2050);
            elseif X{1,i}.north>=10800&&X{1,i}.north<10950
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-2200);
            elseif X{1,i}.north>=10950&&X{1,i}.north<11100
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-2350);
            elseif X{1,i}.north==11100
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=-2500);
            else
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
            end
            
            % plot regression on cross sections
            Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
            figure(i+1)
            plot(xys(Is,2),xys(Is,1),'k-');
            plot(xyd(Id,2),xyd(Id,1),'k-');
            
            clear datains
            clear dataind
            % post 1990 events
            datains(:,1) = [xys(Is,1);Z{1,i}.post1990s];
            datains(:,2) = [xys(Is,2);X{1,i}.post1990s];
            dataind(:,1) = [xyd(Id,1);Z{1,i}.post1990d];
            dataind(:,2) = [xyd(Id,2);X{1,i}.post1990d];
            clear xys
            clear xyd
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xys] = lowess(datains,f,wantplot,imagefile,xdata);
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            Is = find(xdata>=min(datains(:,1))&xdata<=max(datains(:,1)));
            Id = find(xdata>=min(dataind(:,1))&xdata<=max(dataind(:,1)));
            
            if mod(X{1,i}.north/150,2)~=0
                J = find(xdata(Is)>=-3200);
                Is = Is(J);
                Id = [];
            else
            end
            
            % plot regression on cross sections
            plot(xys(Is,2),xys(Is,1),'k--');
            plot(xyd(Id,2),xyd(Id,1),'k--');
            
            % store xyz points for fault nodes on shallow fault
            Xs(numel(Xs)+1:numel(Xs)+numel(Is),1) = xys(Is,2);
            Ys(numel(Ys)+1:numel(Ys)+numel(Is),1) = Y{i}.north;
            Zs(numel(Zs)+1:numel(Zs)+numel(Is),1) = xys(Is,1);
            
            % store xyz points for fault nodes on deep fault
            Xd(numel(Xd)+1:numel(Xd)+numel(Id),1) = xyd(Id,2);
            Yd(numel(Yd)+1:numel(Yd)+numel(Id),1) = Y{i}.north;
            Zd(numel(Zd)+1:numel(Zd)+numel(Id),1) = xyd(Id,1);
        else% shallow fault termination
            % pre 1990 events
            dataind(:,1) = Z{1,i}.pre1990d;
            dataind(:,2) = X{1,i}.pre1990d;
            xdata = depths;
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % vertical extent of fault
            if X{1,i}.north==12600
                Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
                Jd = find(xdata>=-8500&xdata<=-8200);
                Kd = find(xdata>=-12700&xdata<=-12100);
                Id = [Kd;Id;Jd];
            elseif X{1,i}.north==12900
                Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
                Kd = find(xdata==-12100);
                Id = [Kd;Id];
            else
                Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
            end
            
            % plot regression on cross sections
            figure(i+1)
            plot(xyd(Id,2),xyd(Id,1),'k-');
            
            clear dataind
            % post 1990 events
            dataind(:,1) = [xyd(Id,1);Z{1,i}.post1990d];
            dataind(:,2) = [xyd(Id,2);X{1,i}.post1990d];
            clear xyd
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % vertical extent of fault
            Id = find(xdata>=min(dataind(:,1))&xdata<=max(dataind(:,1)));
            
            if mod(X{1,i}.north/150,2)~=0
                Id = [];
            else
            end
            
            % plot regression on cross section
            plot(xyd(Id,2),xyd(Id,1),'k--');
            
            % store xyz points for fault nodes on deep fault
            Xd(numel(Xd)+1:numel(Xd)+numel(Id),1) = xyd(Id,2);
            Yd(numel(Yd)+1:numel(Yd)+numel(Id),1) = Y{i}.north;
            Zd(numel(Zd)+1:numel(Zd)+numel(Id),1) = xyd(Id,1);
        end
    end
end


XX = [Xc;Xs];
YY = [Yc;Ys];
ZZ = [Zc;Zs];

Cz = unique(ZZ);
Cy = unique(YY);

for k=1:50
% Smooth data in y-direction
for i=1:numel(Cz)
    I = find(ZZ==Cz(i));
    [XX(I),r]=smooth(XX(I),0);
end

% Smooth data in z-direction
for i=1:numel(Cy)
    I = find(YY==Cy(i));
    [XX(I),r]=smooth(XX(I),0);
end
end

Xc = XX(1:numel(Xc));
Xs = XX((numel(Xc)+1):(numel(Xc)+numel(Xs)));

Yc = YY(1:numel(Yc));
Ys = YY((numel(Yc)+1):(numel(Yc)+numel(Ys)));

Zc = ZZ(1:numel(Zc));
Zs = ZZ((numel(Zc)+1):(numel(Zc)+numel(Zs)));



% put nodes in 3 column matrix
XYZc(:,1) = Xc;XYZc(:,2) = Yc;XYZc(:,3) = Zc;
XYZs(:,1) = Xs;XYZs(:,2) = Ys;XYZs(:,3) = Zs;
XYZd(:,1) = Xd;XYZd(:,2) = Yd;XYZd(:,3) = Zd;

% connect fault sections
Is = find(XYZc(:,2)==7800&XYZc(:,3)>=-6100);
Id = find(XYZc(:,2)==7800&XYZc(:,3)<=-6100);
XYZs = [XYZc(Is,:);XYZs];
XYZd = [XYZc(Id,:);XYZd];

% triangulate fault nodes
CONc=Delaunay2_5D(XYZc);
CONs=Delaunay2_5D(XYZs);
CONd=Delaunay2_5D(XYZd);

% plot fault mesh on 3D scatterplot
figure(1)
for i=1:numel(CONc(:,1))
    plot3(XYZc([CONc(i,:) CONc(i,1)],1),XYZc([CONc(i,:) CONc(i,1)],2),XYZc([CONc(i,:) CONc(i,1)],3),'b-')
end
for i=1:numel(CONs(:,1))
    plot3(XYZs([CONs(i,:) CONs(i,1)],1),XYZs([CONs(i,:) CONs(i,1)],2),XYZs([CONs(i,:) CONs(i,1)],3),'b-')
end
for i=1:numel(CONd(:,1))
    plot3(XYZd([CONd(i,:) CONd(i,1)],1),XYZd([CONd(i,:) CONd(i,1)],2),XYZd([CONd(i,:) CONd(i,1)],3),'b-')
end

% correct polarity of triangulation
convention = 'z-';
[tnc,nvecc] = PlotTriNormals(CONc,XYZc,convention);
[tns,nvecs] = PlotTriNormals(CONs,XYZs,convention);
[tnd,nvecd] = PlotTriNormals(CONd,XYZd,convention);


%% Translate Faults back to UTM

% north fault
XYZc(:,1) = XYZc(:,1) + xo;
XYZc(:,2) = XYZc(:,2) + yo;
XYZs(:,1) = XYZs(:,1) + xo;
XYZs(:,2) = XYZs(:,2) + yo;
XYZd(:,1) = XYZd(:,1) + xo;
XYZd(:,2) = XYZd(:,2) + yo;


figure(1)
% view(-90,0)
% zlim([-4000 0])
% ylim([6000 12000])