%% MSH_Plot_EQ_3D_southfault.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    2017/05/12
% VERSION:          v.2
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: modified from 'MSH_Plot_EQ_3D_allfaults.m' to
% refine fault mesh.
% 
% INPUT: Seismic catalogs selected from Pacific Northwest Seismic Network:
%           1) FocalMech_south.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-),
% triangulation of fault nodes (tn-), and unit normal vectors of fault element
% triangles (nvec-).
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       fitNormal.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Import Data


%%% south fault %%%
dataSF = importdata('FocalMech_south.csv',',');
for i=2:numel(dataSF)
    d = strsplit(dataSF{i},',');
    datetime = strsplit(d{5});
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{3}) str2num(date{1}) str2num(date{2})];
    dnumSF(i-1,1) = datenum(Date(1),Date(2),Date(3));
    latSF(i-1,1) = str2num(d{6}) + (rand*(10^-7));
    longSF(i-1,1) = str2num(d{7}) + (rand*(10^-7));
    mSF(i-1,1) = str2num(d{8});
    if mSF(i-1,1)<1
        mSF(i-1,1)=1;
    end
    zSF(i-1,1) = -str2num(d{9})*1000 - rand;
end
SSF = 3.^mSF;
[xSF,ySF,utmzone] = deg2utm(latSF,longSF);
% UTM origin
xoSF = 5.68792e5;
yoSF = 5.099634e6;
xSF = xSF - xoSF;
ySF = ySF - yoSF;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xSF;c(:,2) = ySF;c(:,3) = zSF;
n = fitNormal(c, show_graph);
thetaSF = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaSF) sin(thetaSF);-sin(thetaSF) cos(thetaSF)];
for i=1:numel(xSF)
    b = Rcw*[xSF(i);ySF(i)];
%     xSF(i) = b(1)-4000;ySF(i) = b(2)+3000;
xSF(i) = b(1);ySF(i) = b(2);
end
% rotate about y-axis
ddip = -1;
zSF = zSF + 4000;xSF = xSF - [2394.47611389336];
Rcw = [cos(ddip*(pi/180)) sin(ddip*(pi/180));-sin(ddip*(pi/180)) cos(ddip*(pi/180))];
for i=1:numel(xSF)
    b = Rcw*[xSF(i);zSF(i)];
    xSF(i) = b(1);zSF(i) = b(2);
end
zSF = zSF - 4000;xSF = xSF + [2394.47611389336];


%% filter data
% by time

% south fault
JSF = find(dnumSF<datenum(1990,1,1));% 1980s
KSF = find(dnumSF>datenum(1990,1,1));% 1990s & 2000s

%% Plot 3D EQs

% south fault
figure
h1 = scatter3(xSF(JSF),ySF(JSF),zSF(JSF),SSF(JSF),'k');
hold on
h2 = scatter3(xSF(KSF),ySF(KSF),zSF(KSF),SSF(KSF),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 5000])
ylim([-2000 20000])
zlim([-15000 -3000])
xlabel('X(m)','fontsize',12)
ylabel('Y(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('South Fault 1 ',' Strike=',num2str(round(180-thetaSF*(180/pi)))),'fontsize',14)


% depth slices for node locations
depths = [-13000 -10000 -7600 -6400 -5200 -4600 -4000];depths = depths';
% northing coordinates for cross-sections/node locations
norths = [1200:600:18000];% 600m spacing

%% cross sections
for i=1:numel(norths)
        I = find(ySF>(norths(i)-18500)&ySF<=(norths(i)+18500));
        J = find(dnumSF(I)<datenum(1990,1,1));% 1980s
        K = find(dnumSF(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xSF(I(J)),zSF(I(J)),SSF(I(J)),'k');
        hold on
        scatter(xSF(I(K)),zSF(I(K)),SSF(I(K)),'r');
        axis equal;grid on;
        xlim([0 5000])
        ylim([-15000 -3000])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
        XSF{i}.pre1990 = xSF(I(J));XSF{i}.post1990 = xSF(I(K));XSF{i}.north = norths(i);%XEF1{i}.extent = xEF1(Ie);
        YSF{i}.pre1990 = ySF(I(J));YSF{i}.post1990 = ySF(I(K));YSF{i}.north = norths(i);%YEF1{i}.extent = yEF1(Ie);
        ZSF{i}.pre1990 = zSF(I(J));ZSF{i}.post1990 = zSF(I(K));ZSF{i}.north = norths(i);%ZEF1{i}.extent = zEF1(Ie);
end

%% regression on cross-sections
f = 1;% smoothing factor (0 to 1)
wantplot = 0;% zero = no plot
imagefile = 'lowess_plot\';% directory where plot is stored

Xsf=[];
Ysf=[];
Zsf=[];

for i=1:numel(XSF)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
        % pre/post 1990 events
        datain(:,1) = [ZSF{1,i}.pre1990;ZSF{1,i}.post1990];
        datain(:,2) = [XSF{1,i}.pre1990;XSF{1,i}.post1990];
        xdata = depths;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
 
        if mod((XSF{1,i}.north-1200)/600,2)~=0
            I = find(depths>=-5200);
            xdata = depths(I);
        elseif mod((XSF{1,i}.north-1200)/1200,2)~=0
            I = find(depths>=-7600);
            xdata = depths(I);
        else
            xdata = depths;
            I = [1:numel(xdata)];
        end

        % plot regression on cross-sections
        figure(i+1)
        plot(xy(:,2),xy(:,1),'k-');

        % store xyz points for fault nodes
        Xsf(numel(Xsf)+1:numel(Xsf)+numel(I),1) = xy(I,2);
        Ysf(numel(Ysf)+1:numel(Ysf)+numel(I),1) = YSF{i}.north;
        Zsf(numel(Zsf)+1:numel(Zsf)+numel(I),1) = xy(I,1);
end

% % smooth fault in y-direction
% Csf = unique(Zsf);
% for i=1:numel(Csf)
%     I = find(Zsf==Csf(i));
%     [Xsf(I),r]=smooth(Xsf(I),1);
% end

% put nodes in 3 column matrox
XYZsf(:,1) = Xsf;XYZsf(:,2) = Ysf;XYZsf(:,3) = Zsf;

% triangulate fault nodes
CONsf=Delaunay2_5D(XYZsf);

% plot fault mesh on 3Dscatterplot
figure(1)
for i=1:numel(CONsf(:,1))
    plot3(XYZsf([CONsf(i,:) CONsf(i,1)],1),XYZsf([CONsf(i,:) CONsf(i,1)],2),XYZsf([CONsf(i,:) CONsf(i,1)],3),'b-')
end

% Rotate back
Rccw = [cos(thetaSF) -sin(thetaSF);sin(thetaSF) cos(thetaSF)];
for i=1:numel(Xsf)
%     b = Rccw*[(XYZsf(i,1)+4000);XYZsf(i,2)-3000];
    b = Rccw*[XYZsf(i,1);XYZsf(i,2)];
    XYZsf(i,1) = b(1);XYZsf(i,2) = b(2);
end

% correct polarity of triangulation
convention = 'z-';
[tnsf,nvecsf] = PlotTriNormals(CONsf,XYZsf,convention);

%% Translate Faults back to UTM

% south fault
XYZsf(:,1) = XYZsf(:,1) + xoSF;
XYZsf(:,2) = XYZsf(:,2) + yoSF;

figure(1)