%% MSH_Plot_EQ_3D_spudmountainpluton.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: modified from 'MSH_Plot_EQ_3D_allfaults.m' to
% add spud mountain pluton boundary based on surface mapping, gravity and
% MT at depth with the inference that the granitic pluton is more resistive
% than 250 Ohm-meters. Other shallow E-W trending faults were modeled
% in Phase 2 AOIs.
% 
% INPUT: Seismic catalogs selected from Pacific Northwest Seismic Network:
%           1) spud_mountain_pluton_surface.csv
%           2) spud_mountain_pluton_5860m.csv
%           3) spud_mountain_pluton_7860m.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-),
% triangulation of fault nodes (tn-), and unit normal vectors of fault element
% triangles (nvec-).
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       fitNormal.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Import Data

%%% spud mountain pluton points
data1 = importdata('spud_mountain_pluton_surface.csv',',');
x_surf = data1.data(:,1);
y_surf = data1.data(:,1);
z_surf = data1.data(:,1);
data2 = importdata('spud_mountain_pluton_5860m.csv',',');
x_5860 = data2.data(:,1);
y_5860 = data2.data(:,1);
z_5860 = data2.data(:,1);
data3 = importdata('spud_mountain_pluton_7860m.csv',',');
x_7860 = data2.data(:,1);
y_7860 = data2.data(:,1);
z_7860 = data2.data(:,1);
xSMP = [data1.data(:,1);data2.data(:,1);data3.data(:,1)];
ySMP = [data1.data(:,2);data2.data(:,2);data3.data(:,2)];
zSMP = [data1.data(:,3);data2.data(:,3);data3.data(:,3)];
% UTM origin
xoSMP = 550000;
yoSMP = 5123000;
xSMP = xSMP - xoSMP;
ySMP = ySMP - yoSMP;


%% Plot 3D control points

% spud mountain pluton
figure
h1 = plot3(data1.data(:,1)-xoSMP,data1.data(:,2)-yoSMP,data1.data(:,3),'kx');
hold on
h2 = plot3(data2.data(:,1)-xoSMP,data2.data(:,2)-yoSMP,data2.data(:,3),'b.');
plot3(data3.data(:,1)-xoSMP,data3.data(:,2)-yoSMP,data3.data(:,3),'b.');
axis equal;grid on
legend([h1 h2],'surfacepoints','250 ohm-meters')
xlim([0 8000])
ylim([0 16000])
zlim([-8000 3500])
title('Spud Mountain Pluton','fontsize',14)

figure
h1 = plot(data1.data(:,1)-xoSMP,data1.data(:,2)-yoSMP,'kx');
hold on
h2 = plot(data2.data(:,1)-xoSMP,data2.data(:,2)-yoSMP,'b.');
plot(data3.data(:,1)-xoSMP,data3.data(:,2)-yoSMP,'b.');
axis equal;grid on
legend([h1 h2],'surfacepoints','250 ohm-meters')
xlim([0 8000])
ylim([0 16000])
title('Spud Mountain Pluton','fontsize',14)

%% Spud Mountain Pluton

% depth slices for node locations
depths = [-6000 -4800 -4200 -3600 -3300 -3000:150:-1200];depths = depths';
depths_coarse = [-6000:1200:-1200];
depths_medium = [-6000:1200:-3600 -3000:600:-1200];
depths_fine = [-6000 -4800 -4200 -3600 -3300:300:-1200];
% northing coordinates for cross-sections/node locations
norths = [2400 3600 4200 4500:150:13950 14250 14850 15450];

q = 400;% 1/2 window size

for i=1:numel(norths)
        I = find(ySMP>(norths(i)-q)&ySMP<=(norths(i)+q));
        figure
        plot(xSMP(I),zSMP(I),'k.');
        hold on
        axis equal;grid on;
        xlim([0 8000])
        ylim([-8000 3500])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
        X_SMF(i).points = xSMP(I);
        X_SMF(i).north = norths(i);
        Y_SMF(i).points = ySMP(I);
        Y_SMF(i).north = norths(i);
        Z_SMF(i).points = zSMP(I);
        Z_SMF(i).north = norths(i);
end

f = 1;% smoothing factor (0 to 1)
wantplot = 0;% zero = no plot
imagefile = 'lowess_plot\';% directory where plot is stored

X = [];Y = [];Z = [];% fault nodes

for i=1:numel(X_SMF)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
    clear ydata
    

I = find(Z_SMF(1,i).points<-7000);
datain(1,1) = mean(Z_SMF(1,i).points(I));
datain(1,2) = mean(X_SMF(1,i).points(I));
I = find(Z_SMF(1,i).points>-7000&Z_SMF(1,i).points<-2000);
datain(2,1) = mean(Z_SMF(1,i).points(I));
datain(2,2) = mean(X_SMF(1,i).points(I));
I = find(Z_SMF(1,i).points>-2000);
datain(3,1) = mean(Z_SMF(1,i).points(I));
datain(3,2) = mean(X_SMF(1,i).points(I))+500;

    if X_SMF(1,i).north==2400
        xdata = depths_coarse;
    elseif X_SMF(1,i).north==3600
        xdata = depths_medium;
    elseif X_SMF(1,i).north==4200
        xdata = depths_fine;
    elseif X_SMF(1,i).north==14250
        xdata = depths_fine;
    elseif X_SMF(1,i).north==14850
        xdata = depths_medium;
    elseif X_SMF(1,i).north==15450
        xdata = depths_coarse;
    else
        if mod((X_SMF(1,i).north+300)/150,2)~=0
            I = find(depths>=-3000);
            xdata = depths(I);
        elseif mod((X_SMF(1,i).north+300)/300,2)~=0
            I = find(depths>=-3600);
            xdata = depths(I);
        elseif mod((X_SMF(1,i).north+300)/600,2)~=0
            I = find(depths>=-4800);
            xdata = depths(I);
        else
            xdata = depths;
        end
    end
    
    % regression on cross section 
    ydata = interp1(datain(:,1),datain(:,2),xdata,'cubic');
    xy(:,1) = xdata;xy(:,2) = ydata;

    % plot regression on cross section
    figure(i+2)
    plot(xy(:,2),xy(:,1),'k-');

    % store xyz points for fault nodes
    X(numel(X)+1:numel(X)+length(xy),1) = xy(:,2);
    Y(numel(Y)+1:numel(Y)+length(xy),1) = norths(i);
    Z(numel(Z)+1:numel(Z)+length(xy),1) = xy(:,1);
end

% smooth data in y-direction
J = find(Y>9000);
C = unique(Z(J));
for i=1:numel(C)
    I = find(Z(J)==C(i));
    [X(J(I)),r]=smooth(X(J(I)),1);
end
for i=1:numel(C)
    I = find(Z(J)==C(i));
    [X(J(I)),r]=smooth(X(J(I)),1);
end

% put nodes in 3 column matrix
XYZ(:,1) = X;XYZ(:,2) = Y;XYZ(:,3) = Z;

% triangulate fault nodes
CON=Delaunay2_5D(XYZ);

% plot fault mesh on 3D scatterplot
figure(1)
for i=1:numel(CON(:,1))
    plot3(XYZ([CON(i,:) CON(i,1)],1),XYZ([CON(i,:) CON(i,1)],2),XYZ([CON(i,:) CON(i,1)],3),'b-')
end

% correct polarity of triangulation
convention = 'z-';
[tn,nvec] = PlotTriNormals(CON,XYZ,convention);

%% Translate Faults back to UTM

XYZ(:,1) = XYZ(:,1) + xoSMP;
XYZ(:,2) = XYZ(:,2) + yoSMP;






