function [tri,v,nvec,tc,A] = EllipsoidTri(xc,yc,zc,xr,yr,zr,n,trend,plunge,gamma,normal_sign_convention)


% AUTHOR: NC Davatzes
% DATE Created: 2015-02-26
% 
% PURPOSE
%   Generate and triangulate an ellipse with outward pointing normal
%   vectors at their centroids
% 
% INPUTS
%   [xc,yc,zc]  center of the ellipsoid
%   xr          semi-major axis
%   yr          semi-intermediate axis
%   zr          semi-minor axis
%               convention is xr,yr,zr
%   n           number of vertices definining the ellipse is (n+1)^2
%   trend       trend of the semi-major axis
%   plunge      plunge of the semi-major axis
%   alpha       plunge of the semi-intermediate axis
%   gamma       rotation of the semi-intermediate and semi-minor axis about
%               the semi-major axis
%   normal_sign_convention = convention for direction of normals
%               default: 'o+' points out from the center of the ellipsoid
%               alternate: 'o-' points toward ellipse center
%               'r+' or 'r-' points away (toward) the z-axis
%               'z+' or 'z-' points up (down)
% 
% OUTPUTS
%   tri         the correlation matrix grouping vertices to define
%               triangles: it is an m x 3 matrix of row positions of the 
%               vertices defined in the variable "v"; polarity of the
%               related normal vector is determined by the order in which
%               the vertices are listed which is examined to enforce the
%               sign convention designated by "normal_sign_convention"
%   v           m x 3 matrix of vertices defining the ellipsoidal surface
%   nvec        m x 3 matrix, where m is of length(tri) and provides the
%               unit normal vector components of the normal in the 
%               corresponding triangle defined by the corresponding row
%               in tri
%   tc          m x 3 matrix correponsing to tri and nvec defining centroid
%               the centroid of the triangle defined in the corresponding
%               row of tri (or nvec); tc and nvec can be used with quiver3
%               to plot the normal vectors
%   A           structure array containing descriptions of the rotated and
%               translated ellipsoidal axes including:
%               A.A1dir --> a unit vector of direction
%               A.A1 --> a plottable point defining a line from the
%                   ellipsoid center to just outside in the ellipse in the 
%                   direction of each axis
%               A.A1_trend, A.A1_plunge --> the trend and plunge of 
%                   each axis
% 
%   
% NOTES
% 
% 
% 
% 
% EXAMPLE
% % position of ellipsoid
% xc = 100; yc = 0; zc = 0;
% % trend (azimuth) and plunge (positive down) of ellipsoid 
% trend = 30; plunge = 70;
% % Define anisotropy through the semi-axial lengths
% xr = 3; yr = 1; zr = 1;
% % number of vertices definine the ellipsoid will be (n+1)^2
% n = 20;
% figure
% % Generate Ellipoid (default outward pointing normals)
% [tri,v,nvec,tc] = EllipsoidTri(xc,yc,zc,xr,yr,zr,n,trend,plunge);
% % Visualize Resulting plot
% figure
%     h = trisurf(tri,v(:,1),v(:,2),v(:,3)); hold on
%     set(h,'FaceAlpha',0.6,'LineWidth',0.1,'EdgeAlpha',0.7)
%     quiver3(tc(:,1),tc(:,2),tc(:,3),nvec(:,1),nvec(:,2),nvec(:,3),'g','LineWidth',0.5)
%     axis equal, box on, grid on
%     colorbar
%     hold on
%     ha(1) = plot3([xc A.A1(1)],[yc A.A1(2)],[zc A.A1(3)],'r-','LineWidth',2);
%     ha(2) = plot3([xc A.A2(1)],[yc A.A2(2)],[zc A.A2(3)],'k-','LineWidth',2);
%     ha(3) = plot3([xc A.A3(1)],[yc A.A3(2)],[zc A.A3(3)],'b-','LineWidth',2);
%     MyFormat = {'FontSize',14,'FontWeight','bold'};
%     xlabel('x (Easting)',MyFormat{:}), 
%     ylabel('y (Northing)',MyFormat{:}), 
%     zlabel('z (up, elevation)',MyFormat{:})
%     title('Triangulated Ellipsoid Surface',MyFormat{:})
%     legend(ha,'Semi-Major axis',...
%         'Semi-Intermediate axis',...
%         'Semi-Minor axis',...
%         'Location','NorthWest')

%% For purposes of later rotation and translation adopt the following 
%  conventions:
%   - Center at [x,y,z] = 0,0,0];
%   - XR >= YR >= ZR

[X,Y,Z] = ellipsoid(0,0,0,xr,yr,zr,n);

%% GENERATE Triangulated Mesh
% Transform the Matrices X,Y,Z into column vectors suitable for
% triangulation
x = X(:); y = Y(:); z = Z(:);
% Note that for a closed (non-monotonic) volume, triangulation can be
% achieved via 1 of 2 approaches: (1) transform the equation into a
% monotonic equivalent, e.g., via a distance formulation; (2) compute
% voxels (volume tetrahedrons), then find the surface of the volume defined
% with coordination defined by the volume elements which gives triangular 
% faces of the voxels at the surface of the volume, i.e., the triangulated
% surface.
dt = delaunayTriangulation(x,y,z); % produces volume elements
    % dt.Points --> vertices
    % dt.ConnectivityList --> rows of vertices comprising each tetrahedron
% To get the surface triangulation use convex hull on these tetrahedron
tri = convexHull(dt); % defines the coordination of the vertices defining 
                      % the surface triangles
v1 = dt.Points(:,1);
v2 = dt.Points(:,2);
v3 = dt.Points(:,3);

%% Rotate ellipse and related supporting data to trend and plunge:
% Rotate ellipse long axis to desired azimuth, assume a x1 (E), x2 (N), x3
% (Elevation, positive z up) reference frame

% rotate about x-axis to determine attitude of intermediate axis 
aij_x = [1          0       0;
        0   cosd(gamma)     sind(gamma);
        0   -sind(gamma)    cosd(gamma)];


% 1. Rotate about y axis, allowing the volume to be inclined
% angle of inclination is the plunge of the long axis, positive numbers 
% tilt down in the direction of the trend

aij_y = [cosd(plunge)   0   sind(plunge);
          0             1   0;
         -sind(plunge)  0   cosd(plunge)];

% 2. Vertical Axis Rotation
% azimuth, posite CW from North, defines the trend of the structure, the
% default trend is N-S
th  = 90-trend; % this will default the long axis to N-S when azi = 0
aij_z = [cosd(-th)  sind(-th)   0;
        -sind(-th)  cosd(-th)   0;
        0           0           1];

 % Combine the rotations and update vertices
%aij = aij_z*aij_y;
aij = aij_z*aij_y*aij_x;

[v1,v2,v3] = rotate(v1,v2,v3,aij);

% determine attitude of semi-major (A1), semi-int (A2), and semi-minor (A3) 
% axes
A.A1 = [1 0 0]; %[(xr+0.5*xr) 0 0];
A.A2 = [0 1 0]; %[0 (yr+0.5*xr) 0];
A.A3 = [0 0 1]; %[0 0 (zr+0.5*xr)];
[A.A1(1),A.A1(2),A.A1(3)] = rotate(A.A1(1),A.A1(2),A.A1(3),aij);
[A.A2(1),A.A2(2),A.A2(3)] = rotate(A.A2(1),A.A2(2),A.A2(3),aij);
[A.A3(1),A.A3(2),A.A3(3)] = rotate(A.A3(1),A.A3(2),A.A3(3),aij);
% determine trend and plunge of each axis
MyScale = xr+0.3*xr;
% A1 - major axis
A.A1_dir    = A.A1; A.A1 = A.A1*MyScale;
A.A1_plunge = 90-acosd(dot(A.A1_dir,[0 0 -1])); % using unit vectors
A.A1_trend  = atan2d(A.A1_dir(1),A.A1_dir(2));
% A2 - intermediate axis
A.A2_dir    = A.A2; A.A2 = A.A2*MyScale;
A.A2_plunge = 90-acosd(dot(A.A2_dir,[0 0 -1]));
A.A2_trend  = atan2d(A.A2_dir(1),A.A2_dir(2));
% A3 - minor axis
A.A3_dir    = A.A3; A.A3 = A.A3*MyScale;
A.A3_plunge = 90-acosd(dot(A.A3_dir,[0 0 -1]));
A.A3_trend  = atan2d(A.A3_dir(1),A.A3_dir(2));

%% Plot Normals at Vertices to ensure sign convention:
%  normals point outward from ellipsoid surface
%  Thus a positive normal traction (traction in the same direction as the
%  normal) corresponds to a positive pressure and expansion of the elastic
%  body.
%  Proposed Poly3D BC on the elements are: 
%   Td = Tx = 0; Ts = Ty = 0; Tn = Tz = Pm
%  where Pm is an equivalent pressure in the magma chamber and is uniform
%  across the chamber surface defined by the ellipsoid.

% Test normal porlarity and conform to convention
if exist('normal_sign_convention')==0
    normal_sign_convention = 'o+'; 
end
[tri,nvec,tc,ax] = PlotTriNormals(tri,[v1,v2,v3],normal_sign_convention);
linkprop(ax,'CameraPosition')

% Translate position as per xc,yc,zc
% v1 = v1 + xc; v2 = v2 + yc; v3 = v3 + zc; v = [v1,v2,v3];
% tc(:,1) = tc(:,1) + xc; tc(:,2) = tc(:,2) + yc; tc(:,3) = tc(:,3) + zc; 

v = [v1,v2,v3];
xt = [xc,yc,zc];
for i = 1:3
    v(:,i) = v(:,i)+xt(i);
    tc(:,i) = tc(:,i)+xt(i); % Centroids of triangles
    A.A1(i) = A.A1(i)+xt(i); % Axes of ellipse
    A.A2(i) = A.A2(i)+xt(i);
    A.A3(i) = A.A3(i)+xt(i); 
end


