% MASTER_p3d_MSH_v3.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     2/27/2015
% DATE MODIFIED:    5/17/2017
% VERSION:          v.3
%
% PURPOSE:
%   Script to run poly3d, version from 1999, in c++ is slaved to matlab
%   for:
%   - Modeling the faults at Mount Saint Helens
%
% OBSERVATION GRIDS:
%      Areal grids: 2D
%       Extent of AltaRock leases.
%       -AOI north
%       -AOI south
%          
% PURPOSE OF MODIFICATION: To model stress boundary using rotational strain
% component and strains removed from near block boundary. Also, run 200m
% and 2km models back to back.
% 
% v3: Using refined fault mesh, added eastern edge of spud mountain
% plution, and 2 shallow ENE-SWS cross faults in the north and south. Now
% using displacement boundaries on deep faults based on Wells and
% Coppersmith, 1994. Also using two seperate observation grids for north
% and south AOIs.
%
% INPUT: Fault nodes and connectivity in global coordinates NAD_1983_UTM_ZONE_10N
%           -MSH_NorthFault_refined.xls
%           -MSH_SouthFault_refined.xls
%           -MSH_NorthCrossfault.xls
%           -MSH_SouthCrossfault.xls
%           -MSH_SpudMountain.xls
%       Stress tensors for 200m and 2000m depths
%           -StressTensorMSH200.csv
%           -StressTensorMSH2km.csv
% 
% OUTPUT: Poly 3D Input and Output files named:
%           -MtStHelens200m.in
%           -MtStHelens200m.out
%           -MtStHelens2km.in
%           -MtStHelens2km.out
%
% NOTE: 
% - Based on original files drafted by J. Ole Kaven
% 
% - Note specific paths to the folder containg p3d and to the folder
% containing this mfile are specified for running p3d and saving .in and
% .out files respectively
% 
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3din_FAULT_func_v1_200m
% - p3din_FAULT_func_v1_2km
% - EllipsoidTri.m
% - rotate.m

clear all, close all



%% 1. Define OBSERVATION GRID RANGES
% Generate observation grids in global coordinates.

% AREAL GRIDS: 2D

% Extent of northern AOI
% User Coordinate System
x1o = 553150;
x2o = 5126600;
x3o = 0;
rot1 = 0;
rot2 = 0;
rot3 = 0;
CS_AOInorth = [x1o, x2o, x3o, rot1, rot2, rot3, 123];% matrix used in p3din_FAULT_func_v1
% min/max values 2000ft spacing at 200m depth
minx = 0;  % min value of x
maxx = 9300;  % max value of x
miny = 0;  % min value of y
maxy = 10650;  % max value of y
minz = -200;  % min value of z
maxz = -200;  % max value of z    
npx  = 63; % number of points corresponding to inc in each dimension
npy  = 71; % see also function: linspace
npz  = 1;
OBSgridnorth200 = [minx miny minz maxx maxy maxz npx npy npz];% matrix used in p3din_FAULT_func_v1
% min/max values 2000ft spacing at 3000m depth
minx = 0;  % min value of x
maxx = 9300;  % max value of x
miny = 0;  % min value of y
maxy = 10650;  % max value of y
minz = -2000;  % min value of z
maxz = -2000;  % max value of z
npx  = 63; % number of points corresponding to inc in each dimension
npy  = 71; % see also function: linspace
npz  = 1;
OBSgridnorth2km = [minx miny minz maxx maxy maxz npx npy npz];% matrix used in p3din_FAULT_func_v1

% Extent of southern AOI
% User Coordinate System
x1o = 561000;
x2o = 5101600;
x3o = 0;
rot1 = 0;
rot2 = 0;
rot3 = 0;
CS_AOIsouth = [x1o, x2o, x3o, rot1, rot2, rot3, 123];% matrix used in p3din_FAULT_func_v1
% min/max values 2000ft spacing at 200m depth
minx = 0;  % min value of x
maxx = 6900;  % max value of x
miny = 0;  % min value of y
maxy = 7350;  % max value of y
minz = -200;  % min value of z
maxz = -200;  % max value of z    
npx  = 47; % number of points corresponding to inc in each dimension
npy  = 50; % see also function: linspace
npz  = 1;
OBSgridsouth200 = [minx miny minz maxx maxy maxz npx npy npz];% matrix used in p3din_FAULT_func_v1
% min/max values 2000ft spacing at 3000m depth
minx = 0;  % min value of x
maxx = 6900;  % max value of x
miny = 0;  % min value of y
maxy = 7350;  % max value of y
minz = -2000;  % min value of z
maxz = -2000;  % max value of z
npx  = 47; % number of points corresponding to inc in each dimension
npy  = 50; % see also function: linspace
npz  = 1;
OBSgridsouth2km = [minx miny minz maxx maxy maxz npx npy npz];% matrix used in p3din_FAULT_func_v1

%% 2. Define Boundary Conditions to run

% depth to water table(meters)
dw = 20;

% import stress tensor(s)
% 200m
Sigma200 = importdata('StressTensorMSH200.csv',',');% Stress Tensor
d = .2;% depth km
Pp200 = (9.733*(d-(dw/1000)));
Pf200 = Pp200*1000000;% fluid pressure used in poly3D
% 2000m
Sigma2km = importdata('StressTensorMSH2km.csv',',');% Stress Tensor
d = 2;% depth km
Pp2km = (9.733*(d-(dw/1000)));
Pf2km = Pp2km*1000000;% fluid pressure used in poly3D

% Elastic Properties
nu = 0.2905;% poisson's ratio from Vp/Vs ratio between granitic plutons
E = 20000000000; % Young's Modulus in Pa
       
%% 3. IMPORT FAULTS
    
    % North Fault
    XYZc = xlsread('MSH_NorthFault_refined.xls','XYZc','A2:C5000');
    tnc = xlsread('MSH_NorthFault_refined.xls','tnc','A1:C5000');
    XYZs = xlsread('MSH_NorthFault_refined.xls','XYZs','A2:C5000');
    tns = xlsread('MSH_NorthFault_refined.xls','tns','A1:C5000');
    XYZd = xlsread('MSH_NorthFault_refined.xls','XYZd','A2:C5000');
    tnd = xlsread('MSH_NorthFault_refined.xls','tnd','A1:C5000');

    % Spud Mountain Pluton
    XYZsmp = xlsread('MSH_SpudMountain.xls','XYZsmp','A2:C5000');
    tnsmp = xlsread('MSH_SpudMountain.xls','tnsmp','A1:C5000');
    
    % North Crossfault
    XYZncf = xlsread('MSH_NorthCrossfault.xls','XYZncf','A2:C5000');
    tnncf = xlsread('MSH_NorthCrossfault.xls','tnncf','A1:C5000');
    
    % South Fault
    XYZsf = xlsread('MSH_SouthFault_refined.xls','XYZsf','A2:C5000');
    tnsf = xlsread('MSH_SouthFault_refined.xls','tnsf','A1:C5000');
    
    % South Crossfault
    XYZscf = xlsread('MSH_SouthCrossfault','XYZscf','A2:C5000');
    tnscf = xlsread('MSH_SouthCrossfault','tnscf','A1:C5000');
    
%% 4. Create Magma Chamber Ellipsoid
    
% distances given in meters
% position of ellipsoid
LATmc = 46.204;LONmc = -122.187;
[Xmc,Ymc,utmzone] = deg2utm(LATmc,LONmc);
xc = Xmc; yc = Ymc; zc = -9000;
% trend (azimuth) and plunge (positive down) of ellipsoid; gamma is the
% ortation about the semi-major axis to allow rotation of the 
% semi-intermediate and semi-minor axes out of horizontal and vertical
% respectively
trend = 0; plunge = 90; gamma = 0;
% Define anisotropy through the semi-axial lengths
xr = 3000; yr = 1000; zr = 1000;
% number of vertices definine the ellipsoid will be (n+1)^2
n = 15;

% Generate Ellipoid (default outward pointing normals)
[tri,v,nvec,tc,A] = EllipsoidTri(xc,yc,zc,xr,yr,zr,n,trend,plunge,gamma);

XYZmc = v;tnmc = tri;

%% 5. Magma Chamber Boundary Conditions
%  Proposed Poly3D BC on the elements are: 
%   Td = Tx = 0; Ts = Ty = 0; Tn = Tz = Pm
%  where Pm is an equivalent pressure in the magma chamber. and dPm is the
%  difference from the ambient stress field such that positive dPM is 
%  pressure in excess of ambient and negative values are a reudction
%  (suction)
%  and dPm represents a traction everywhere on the chamber surface 
%  defined by the ellipsoid.

%   dPm   derivation:
%       DERIVATION 1:
%       dPm = (rho_r - rho_m)zg; 
%       rho_r = country rock density = 2700 kg.m^-3 (Williams et al, 1987)
%       rho_m = magma density = 2460 km.m^-3 (Olhoeft et al, 1981)

%   Use Pm = rho_m*z*g for normal tractions
Pm200 = -(2460*200*9.81);
Pm2km = -(2460*2000*9.81);

%% 6. RUN p3din_FAULT_v1.m to generate P3D input file and run P3D

tic

    filename = 'MtStHelens200m';
    path_in = strcat('InputOutputFiles/');
    filename_in  = [path_in,filename,'.in'];
    
        % p3din_FAULT_v1
        p3din_FAULT_func_v1_MSH_200m(E,nu,Sigma200,Pf200,filename_in,XYZc,XYZs,XYZd,XYZsmp,XYZncf,XYZsf,XYZscf,XYZmc,tnc,tns,tnd,tnsmp,tnncf,tnsf,tnscf,tnmc,Pm200,CS_AOInorth,CS_AOIsouth,OBSgridnorth200,OBSgridsouth200)
    
%         %%% Un-comment these lines to run Poly3D %%%
%         filename_out  = [path_in,filename,'.out'];
%         pcdloc = '/P3D/P3DinC1999/newpoly3d';
%         system([pcdloc,' -i ',filename_in,' -o ',filename_out])

toc
tic

    filename = 'MtStHelens2km';
    path_in = strcat('InputOutputFiles/');
    filename_in  = [path_in,filename,'.in'];
    
        % p3din_FAULT_v1
        p3din_FAULT_func_v1_MSH_2km(E,nu,Sigma2km,Pf2km,filename_in,XYZc,XYZs,XYZd,XYZsmp,XYZncf,XYZsf,XYZscf,XYZmc,tnc,tns,tnd,tnsmp,tnncf,tnsf,tnscf,tnmc,Pm2km,CS_AOInorth,CS_AOIsouth,OBSgridnorth2km,OBSgridsouth2km)
    
%         %%% Un-comment these lines to run Poly3D %%%
%         filename_out  = [path_in,filename,'.out'];
%         pcdloc = '/P3D/P3DinC1999/newpoly3d';
%         system([pcdloc,' -i ',filename_in,' -o ',filename_out])

toc