% MSH_PermeabilityModel_200m_AOIs.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     8/24/2015
% DATE MODIFIED:    5/22/2017
% VERSION:          v.2
%
% PURPOSE:
%   Read Poly3D .out file and process data for permeability potential
%         
% PURPOSE OF MODIFICATION: using less faults, and finer fault mesh and observation grids
% around the areas of interest for phase 2 of Washington play fairway
% analysis. Also, combining Sigma3 and dilation tendency layers into
% "dilation" layer, and removing displacement layer. only four favorability
% layers instead of eight.
%
% INPUT: Poly3D output file
%           MtStHelens200m.out
% 
% OUTPUT:
%   fault/stress model outputs:
%           -FAULT_GRID: x, y, z, slip tendency, dilation tendency, displacement(m), displacement gradient(m/m)
%           -STRESS_GRID: x, y, z, maximum Coulomb shear stress(Pa), sigma3(Pa)
%   Permeability potential layers:
%           -Fault2.sliptendency: x, y, z, slip tendency
%           -Fault2.displacementgradient: x, y, z, normalized displacement gradient
%           -Stress2.MaximumCoulombShearStress: x, y, z, normalized maximum Coulomb shear stress
%           -Stress2.dilation: x, y, z, normalized sigma 3 + dilation tendency
%
% NOTE: 
%
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3doutputreader_v05
% - redblue

clear all;close all

filename = 'InputOutputFiles\MtStHelens2km.out';

out = p3doutputreader_v05(filename);
% Generates structured array including
%   FAULT OBJECTS
%   out.fault.xyz   = array of vertex locations
% 
%   out.fault.vert  = array of vertices
%                     [VERTNUM X1 X2 X3]
% 
%   out.fault.disp  = cell array contain displacements for each fault obj
%                     [ELT X1C X2C X3C B1 U1(+) U1(-) B2 U2(+) U2(-) B3 U3(+)]
%                     ELT = element number, for use in tri
%                     X1C, X2C, X3C = element centroid coordinates
%                     B1, B2, B3 are the burgers vectos components as
%                       (strike,dip,normal)
%                     Ui(+), Ui(-) = relative displacment at centroid
%
%   out.fault.tract = cell array contain tractions for each fault obj
%                     [ELT X1C X2C X3C T1 T2 T3]
%                     T1,T2,T3 are the traction components
%                       (strike,dip,normal)
%   out.fault.name  = cell array contain displacements for each fault obj
% 
%
%   OBSERVATION GRID
%   out.obs.xyz     = array containing the obs grid pts [X1,X2,X3]
%
%   out.obs.disp    = array containing displacements at obs pts
%                     [X1 X2 X3 U1 U2 U3]
%                     U1,U2,U3 = components of the displacement vector
%                     
%   out.obs.pstress = array containing principal stress and principal
%                     stress direcitons at obs pts
%                     [X1 X2 X3 N1 N2 N3 SIG1 N1 N2 N3 SIG2 N1 N2 N3 SIG3]
%                     SIG1,2,3 = principal stress components (tensoral
%                     convention) at each observation point
%                     N1,N2,N3 = principal stress direction vectors for the
%                       following principal stress in the array)
%
%   out.obs.stress  = array contain 
%                     [X1 X2 X3 SIG11 SIG22 SIG33 SIG12 SIG23 SIG13]
%                     SIG11,22,33,12,23,13 = stress tensor component at
%                     each observation point

Sigma2km = importdata('StressTensorMSH2km.csv',',');
[V,PS2km] = eig(Sigma2km);% PS2km = eigenvalues as principal stresses

%% Slip/Dilation Tendency/Displacement/Displacement Gradient

%%% 2km %%%

% North Fault Coarse
nvecc = xlsread('MSH_NorthFault_refined.xls','nvecc','A2:C5000');
I = find(out.fault.disp{1,1}(:,4)<-1900&out.fault.disp{1,1}(:,4)>-2200);
% Tractions
clear XYZ
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
for i=1:length(I)
    XYZ(i,:) = out.fault.disp{1,1}(I(i),2:4);% XYZ points
    T(i,:) = Sigma2km*nvecc(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecc(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STc(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTc(i,1) = ((-PS2km(1,1))-(Tn(i,1)))/((-PS2km(1,1))-(-PS2km(3,3)));% Dilation Tendencies
    Dispc(i,1) = (out.fault.disp{1,1}(I(i),5)^2 + out.fault.disp{1,1}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STc = STc(I,1);
DTc = DTc(I,1);
Dispc = Dispc(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispc(i)/100;
    else
        dg(i,1) = abs(Dispc(i)-Dispc(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispc(i)/100;
for i=1:length(I)
    DGc(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
n=1;
clear Dobs;clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
%     J(i,1) = find(Dobs==min(Dobs));
%     J(i,2) = i;
    if min(Dobs)<=150
        J(n,1) = find(Dobs==min(Dobs));
        J(n,2) = i;
        n=n+1;
    else
    end
end
% Find unique observation points and closest element center
Lc(:,1) = unique(J(:,1));
for i=1:length(Lc)
    I = find(J(:,1)==Lc(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out.obs.pstress{1}(Lc(i),1))^2 + (XYZ(J(I(j),2),2)-out.obs.pstress{1}(Lc(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lc(i,2) = J(I(K),2);
end

% North Fault Shallow (Spirit Lake Pluton)
nvecs = xlsread('MSH_NorthFault_refined.xls','nvecs','A2:C5000');
I = find(out.fault.disp{1,2}(:,4)<-1900&out.fault.disp{1,2}(:,4)>-2200);
% Tractions
clear XYZ
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
for i=1:length(I)
    XYZ(i,:) = out.fault.disp{1,2}(I(i),2:4);% XYZ points
    T(i,:) = Sigma2km*nvecs(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecs(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STs(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTs(i,1) = ((-PS2km(1,1))-(Tn(i,1)))/((-PS2km(1,1))-(-PS2km(3,3)));% Dilation Tendencies
    Disps(i,1) = (out.fault.disp{1,2}(I(i),5)^2 + out.fault.disp{1,2}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STs = STs(I,1);
DTs = DTs(I,1);
Disps = Disps(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Disps(i)/100;
    else
        dg(i,1) = abs(Disps(i)-Disps(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Disps(i)/100;
for i=1:length(I)
    DGs(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
n=1;
clear Dobs;clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
%     J(i,1) = find(Dobs==min(Dobs));
%     J(i,2) = i;
    if min(Dobs)<=150
        J(n,1) = find(Dobs==min(Dobs));
        J(n,2) = i;
        n=n+1;
    else
    end
end
% Find unique observation points and closest element center
Lsh(:,1) = unique(J(:,1));
for i=1:length(Lsh)
    I = find(J(:,1)==Lsh(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out.obs.pstress{1}(Lsh(i),1))^2 + (XYZ(J(I(j),2),2)-out.obs.pstress{1}(Lsh(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lsh(i,2) = J(I(K),2);
end

% Spud Mountain Pluton
nvecsmp = xlsread('MSH_SpudMountain.xls','nvecsmp','A2:C5000');
I = find(out.fault.disp{1,4}(:,4)<-1900&out.fault.disp{1,4}(:,4)>-2200);
% Tractions
clear XYZ
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
for i=1:length(I)
    XYZ(i,:) = out.fault.disp{1,4}(I(i),2:4);% XYZ points
    T(i,:) = Sigma2km*nvecsmp(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecsmp(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STsmp(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTsmp(i,1) = ((-PS2km(1,1))-(Tn(i,1)))/((-PS2km(1,1))-(-PS2km(3,3)));% Dilation Tendencies
    Dispsmp(i,1) = (out.fault.disp{1,4}(I(i),5)^2 + out.fault.disp{1,4}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STsmp = STsmp(I,1);
DTsmp = DTsmp(I,1);
Dispsmp = Dispsmp(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispsmp(i)/100;
    else
        dg(i,1) = abs(Dispsmp(i)-Dispsmp(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispsmp(i)/100;
for i=1:length(I)
    DGsmp(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
n=1;
clear Dobs;clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
%     J(i,1) = find(Dobs==min(Dobs));
%     J(i,2) = i;
    if min(Dobs)<=150
        J(n,1) = find(Dobs==min(Dobs));
        J(n,2) = i;
        n=n+1;
    else
    end
end
% Find unique observation points and closest element center
Lsmp(:,1) = unique(J(:,1));
for i=1:length(Lsmp)
    I = find(J(:,1)==Lsmp(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out.obs.pstress{1}(Lsmp(i),1))^2 + (XYZ(J(I(j),2),2)-out.obs.pstress{1}(Lsmp(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lsmp(i,2) = J(I(K),2);
end

% North Crossfault
nvecncf = xlsread('MSH_NorthCrossfault.xls','nvecncf','A2:C5000');
I = find(out.fault.disp{1,5}(:,4)<-1973&out.fault.disp{1,5}(:,4)>-2114);
% Tractions
clear XYZ
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
for i=1:length(I)
    XYZ(i,:) = out.fault.disp{1,5}(I(i),2:4);% XYZ points
    T(i,:) = Sigma2km*nvecncf(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecncf(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STncf(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTncf(i,1) = ((-PS2km(1,1))-(Tn(i,1)))/((-PS2km(1,1))-(-PS2km(3,3)));% Dilation Tendencies
    Dispncf(i,1) = (out.fault.disp{1,5}(I(i),5)^2 + out.fault.disp{1,5}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STncf = STncf(I,1);
DTncf = DTncf(I,1);
Dispncf = Dispncf(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispncf(i)/100;
    else
        dg(i,1) = abs(Dispncf(i)-Dispncf(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispncf(i)/100;
for i=1:length(I)
    DGncf(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs;clear J
n=1;
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
%     J(i,1) = find(Dobs==min(Dobs));
%     J(i,2) = i;
    if min(Dobs)<=150
        J(n,1) = find(Dobs==min(Dobs));
        J(n,2) = i;
        n=n+1;
    else
    end
end
% Find unique observation points and closest element center
Lncf(:,1) = unique(J(:,1));
for i=1:length(Lncf)
    I = find(J(:,1)==Lncf(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out.obs.pstress{1}(Lncf(i),1))^2 + (XYZ(J(I(j),2),2)-out.obs.pstress{1}(Lncf(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lncf(i,2) = J(I(K),2);
end

% South Crossfault
nvecscf = xlsread('MSH_SouthCrossfault.xls','nvecscf','A2:C5000');
I = find(out.fault.disp{1,7}(:,4)<-1973&out.fault.disp{1,7}(:,4)>-2114);
% Tractions
clear XYZ
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
for i=1:length(I)
    XYZ(i,:) = out.fault.disp{1,7}(I(i),2:4);% XYZ points
    T(i,:) = Sigma2km*nvecscf(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecscf(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STscf(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTscf(i,1) = ((-PS2km(1,1))-(Tn(i,1)))/((-PS2km(1,1))-(-PS2km(3,3)));% Dilation Tendencies
    Dispscf(i,1) = (out.fault.disp{1,7}(I(i),5)^2 + out.fault.disp{1,7}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STscf = STscf(I,1);
DTscf = DTscf(I,1);
Dispscf = Dispscf(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispscf(i)/100;
    else
        dg(i,1) = abs(Dispscf(i)-Dispscf(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispscf(i)/100;
for i=1:length(I)
    DGscf(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs;clear J
n=1;
for i=1:length(I)
    for j=1:length(out.obs.pstress{2})
        Dobs(j) = ((XYZ(i,1)-out.obs.pstress{2}(j,1))^2 + (XYZ(i,2)-out.obs.pstress{2}(j,2))^2)^.5;
    end
    if min(Dobs)<=150
        J(n,1) = find(Dobs==min(Dobs));
        J(n,2) = i;
        n=n+1;
    else
    end
end
% Find unique observation points and closest element center
Lscf(:,1) = unique(J(:,1));
for i=1:length(Lscf)
    I = find(J(:,1)==Lscf(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out.obs.pstress{2}(Lscf(i),1))^2 + (XYZ(J(I(j),2),2)-out.obs.pstress{2}(Lscf(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lscf(i,2) = J(I(K),2);
end

%% combine fault layers
% north
Ln = [Lc;Lsh;Lsmp;Lncf];
STalln = [STc(Lc(:,2));STs(Lsh(:,2));STsmp(Lsmp(:,2));STncf(Lncf(:,2))];
DTalln = [DTc(Lc(:,2));DTs(Lsh(:,2));DTsmp(Lsmp(:,2));DTncf(Lncf(:,2))];
Dispalln = [Dispc(Lc(:,2));Disps(Lsh(:,2));Dispsmp(Lsmp(:,2));Dispncf(Lncf(:,2))];
DGalln = [DGc(Lc(:,2));DGs(Lsh(:,2));DGsmp(Lsmp(:,2));DGncf(Lncf(:,2))];
% south
Ls = [Lscf];
STalls = [STscf(Lscf(:,2))];
DTalls = [DTscf(Lscf(:,2))];
Dispalls = [Dispscf(Lscf(:,2))];
DGalls = [DGscf(Lscf(:,2))];

% put fault layers on map coordinates
% north
k=1;
for i=1:length(out.obs.pstress{1})
    FAULT_GRIDn(i,1:3) = out.obs.pstress{1}(i,1:3);
    if find(Ln(:,1)==i)
%         if STalln(k)>1
%             ST(i,1) = 1;
%         elseif STalln(k)<0
%             ST(i,1) = 0;
%         else
            ST(i,1) = STalln(k);
%         end
        DT(i,1) = DTalln(k);
        Disp(i,1) = Dispalln(k);
        DG(i,1) = DGalln(k);
        k=k+1;
    else
        ST(i,1) = 0;
        DT(i,1) = 0;
        Disp(i,1) = 0;
        DG(i,1) = 0;
    end
end
% FAULT_GRID --> X,Y,Z,slip tendency,dilation tendency,displacement,displacement gradient
FAULT_GRIDn(:,4) = ST;
FAULT_GRIDn(:,5) = DT;
% FAULT_GRIDn(:,6) = Disp;
FAULT_GRIDn(:,6) = DG;

% south
clear ST;clear DT;clear Disp;clear DG
k=1;
for i=1:length(out.obs.pstress{2})
    FAULT_GRIDs(i,1:3) = out.obs.pstress{2}(i,1:3);
    if find(Ls(:,1)==i)
        if STalls(k)>1
            ST(i,1) = 1;
        elseif STalls(k)<0
            ST(i,1) = 0;
        else
            ST(i,1) = STalls(k);
        end
        DT(i,1) = DTalls(k);
        Disp(i,1) = Dispalls(k);
        DG(i,1) = DGalls(k);
        k=k+1;
    else
        ST(i,1) = 0;
        DT(i,1) = 0;
        Disp(i,1) = 0;
        DG(i,1) = 0;
    end
end
% FAULT_GRID --> X,Y,Z,slip tendency,dilation tendency,displacement,displacement gradient
FAULT_GRIDs(:,4) = ST;
FAULT_GRIDs(:,5) = DT;
% FAULT_GRIDs(:,6) = Disp;
FAULT_GRIDs(:,6) = DG;

%% Combine Stress layers

% north
% calculate Sc and S3
for i=1:length(out.obs.pstress{1})
    Scn(i,1) = (((-out.obs.pstress{1}(i,15))-(-out.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{1}(i,15))+(-out.obs.pstress{1}(i,7)))/2);
    S3n(i,1) = (-out.obs.pstress{1}(i,7));
    if find(Ln(:,1)==i)
        Scn(i,1) = 0;
        S3n(i,1) = 0;
    end
end

% extract stress values to compute means and standard deviations
k=1;
for i=1:length(Scn)
    if isnan(Scn(i))
    elseif Scn(i)>1e11
    else
        ScRealn(k,1) = Scn(i);
        S3Realn(k,1) = S3n(i);
        k=k+1;
    end
end

% Mean and Standard Deviation
% Sc
Scmeann = mean(ScRealn);
SigScn = std(ScRealn);
% S3
S3meann = mean(S3Realn);
SigS3n = std(S3Realn);

% STRESS_GRID --> X,Y,Z,maximum Coulomb shear stress,sigma3
STRESS_GRIDn(:,1:3) = out.obs.pstress{1}(:,1:3);
STRESS_GRIDn(:,4) = Scn;
STRESS_GRIDn(:,5) = S3n;

% south
% calculate Sc and S3
for i=1:length(out.obs.pstress{2})
    Scs(i,1) = (((-out.obs.pstress{2}(i,15))-(-out.obs.pstress{2}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{2}(i,15))+(-out.obs.pstress{2}(i,7)))/2);
    S3s(i,1) = (-out.obs.pstress{2}(i,7));
    if find(Ls(:,1)==i)
        Scs(i,1) = 0;
        S3s(i,1) = 0;
    end
end

% extract stress values to compute means and standard deviations
k=1;
for i=1:length(Scs)
    if isnan(Scs(i))
    elseif Scs(i)>1e9
    else
        ScReals(k,1) = Scs(i);
        S3Reals(k,1) = S3s(i);
        k=k+1;
    end
end

% Mean and Standard Deviation
% Sc
Scmeans = mean(ScReals);
SigScs = std(ScReals);
% S3
S3means = mean(S3Reals);
SigS3s = std(S3Reals);

% STRESS_GRID --> X,Y,Z,maximum Coulomb shear stress,sigma3
STRESS_GRIDs(:,1:3) = out.obs.pstress{2}(:,1:3);
STRESS_GRIDs(:,4) = Scs;
STRESS_GRIDs(:,5) = S3s;

%% Permeability Model Layer Normalization

% north
% Fault Layers
% no normalization needed for slip/dilation tendency (0-1)
Fault.sliptendency_north(:,1:3) = FAULT_GRIDn(:,1:3);
Fault.sliptendency_north(:,4) = FAULT_GRIDn(:,4);
Fault.dilationtendency_north(:,1:3) = FAULT_GRIDn(:,1:3);
Fault.dilationtendency_north(:,4) = FAULT_GRIDn(:,5);
% % displacement
% Fault.displacement_north(:,1:3) = FAULT_GRIDn(:,1:3);
% Fault.displacement_north(:,4) = FAULT_GRIDn(:,6)/max(FAULT_GRIDn(:,6));
% displacement gradient
Fault.displacementgradient_north(:,1:3) = FAULT_GRIDn(:,1:3);
Fault.displacementgradient_north(:,4) = FAULT_GRIDn(:,6)/max(FAULT_GRIDn(:,6));

I  = find(Fault.displacementgradient_north(:,4)==max(Fault.displacementgradient_north(:,4)));
Fault.displacementgradient_north(I,4) = .15;
Fault.displacementgradient_north(:,4) = Fault.displacementgradient_north(:,4)/max(Fault.displacementgradient_north(:,4));


% normalization of stress between +/- X standard deviations
nsd = 3;% # of standard deviations

% Stress Layers
Stress.MaximumCoulombShearStress_north(:,1:3) = STRESS_GRIDn(:,1:3);
Stress.MaximumCoulombShearStress_north(:,4) = (STRESS_GRIDn(:,4)-(Scmeann-nsd*SigScn))/(2*nsd*SigScn);
I = find(Stress.MaximumCoulombShearStress_north(:,4)<0);
Stress.MaximumCoulombShearStress_north(I,4) = 0;
I = find(Stress.MaximumCoulombShearStress_north(:,4)>1);
Stress.MaximumCoulombShearStress_north(I,4) = 1;
for i=1:length(Ln)
    Stress.MaximumCoulombShearStress_north(Ln(i,1),4) = 0;
end

% Inverse Shmin values by adding 2*mean to negative values
Stress.Sigma3_north(:,1:3) = STRESS_GRIDn(:,1:3);
Stress.Sigma3_north(:,4) = -STRESS_GRIDn(:,5);
Stress.Sigma3_north(:,4) = (Stress.Sigma3_north(:,4)-(-S3meann-nsd*SigS3n))/(2*nsd*SigS3n);
I = find(Stress.Sigma3_north(:,4)<0);
Stress.Sigma3_north(I,4) = 0;
I = find(Stress.Sigma3_north(:,4)>1);
Stress.Sigma3_north(I,4) = 1;
for i=1:length(Ln)
    Stress.Sigma3_north(Ln(i,1),4) = 0;
end

% new favorability layers
Fault2.north.sliptendency = Fault.sliptendency_north;
Fault2.north.slipgradient = Fault.displacementgradient_north;
Stress2.north.shear = Stress.MaximumCoulombShearStress_north;
Stress2.north.dilation(:,1:3) = STRESS_GRIDn(:,1:3);
Stress2.north.dilation(:,4) = Stress.Sigma3_north(:,4) + Fault.dilationtendency_north(:,4);

% south
% Fault Layers
% no normalization needed for slip/dilation tendency (0-1)
Fault.sliptendency_south(:,1:3) = FAULT_GRIDs(:,1:3);
Fault.sliptendency_south(:,4) = FAULT_GRIDs(:,4);
Fault.dilationtendency_south(:,1:3) = FAULT_GRIDs(:,1:3);
Fault.dilationtendency_south(:,4) = FAULT_GRIDs(:,5);
% % displacement
% Fault.displacement_south(:,1:3) = FAULT_GRIDs(:,1:3);
% Fault.displacement_south(:,4) = FAULT_GRIDs(:,6)/max(FAULT_GRIDs(:,6));
% displacement gradient
Fault.displacementgradient_south(:,1:3) = FAULT_GRIDs(:,1:3);
Fault.displacementgradient_south(:,4) = FAULT_GRIDs(:,6)/max(FAULT_GRIDs(:,6));

% normalization of stress between +/- X standard deviations
nsd = 3;% # of standard deviations

% Stress Layers
Stress.MaximumCoulombShearStress_south(:,1:3) = STRESS_GRIDs(:,1:3);
Stress.MaximumCoulombShearStress_south(:,4) = (STRESS_GRIDs(:,4)-(Scmeans-nsd*SigScs))/(2*nsd*SigScs);
I = find(Stress.MaximumCoulombShearStress_south(:,4)<0);
Stress.MaximumCoulombShearStress_south(I,4) = 0;
I = find(Stress.MaximumCoulombShearStress_south(:,4)>1);
Stress.MaximumCoulombShearStress_south(I,4) = 1;
for i=1:length(Ls)
    Stress.MaximumCoulombShearStress_south(Ls(i,1),4) = 0;
end

% Inverse Shmin values by adding 2*mean to negative values
Stress.Sigma3_south(:,1:3) = STRESS_GRIDs(:,1:3);
Stress.Sigma3_south(:,4) = -STRESS_GRIDs(:,5);
Stress.Sigma3_south(:,4) = (Stress.Sigma3_south(:,4)-(-S3means-nsd*SigS3s))/(2*nsd*SigS3s);
I = find(Stress.Sigma3_south(:,4)<0);
Stress.Sigma3_south(I,4) = 0;
I = find(Stress.Sigma3_south(:,4)>1);
Stress.Sigma3_south(I,4) = 1;
for i=1:length(Ls)
    Stress.Sigma3_south(Ls(i,1),4) = 0;
end

% new favorability layers
Fault2.south.sliptendency = Fault.sliptendency_south;
Fault2.south.slipgradient = Fault.displacementgradient_south;
Stress2.south.shear = Stress.MaximumCoulombShearStress_south;
Stress2.south.dilation(:,1:3) = STRESS_GRIDs(:,1:3);
Stress2.south.dilation(:,4) = Stress.Sigma3_south(:,4) + Fault.dilationtendency_south(:,4);

%% FIGURES

% north
figure
plot([1:numel(Scn)],Scn,'k-');ylabel('Sc')
hold on
plot([0 numel(Scn)],[Scmeann+nsd*SigScn Scmeann+nsd*SigScn])
plot([0 numel(Scn)],[Scmeann-nsd*SigScn Scmeann-nsd*SigScn])
figure
plot([1:numel(S3n)],S3n,'k-');ylabel('S3')
hold on
plot([0 numel(S3n)],[S3meann+nsd*SigS3n S3meann+nsd*SigS3n])
plot([0 numel(S3n)],[S3meann-nsd*SigS3n S3meann-nsd*SigS3n])
title('north AOI')

figure
plot([1:numel(Stress.MaximumCoulombShearStress_north(:,4))],Stress.MaximumCoulombShearStress_north(:,4),'k.');ylabel('Sc favorability')
figure
plot([1:numel(Stress.Sigma3_north(:,4))],Stress.Sigma3_north(:,4),'k.');ylabel('S3 favorability')


% grid data for contour plots
npx  = 63;
npy  = 71;
for j=0:(npy-1)
for i=0:(npx-1)
    Xn(1+i,1+j) = out.obs.pstress{1}((1+i)+npx*j,1);
    Yn(1+i,1+j) = out.obs.pstress{1}((1+i)+npx*j,2);
    ScXYn(1+i,1+j) = ((((-out.obs.pstress{1}((1+i)+npx*j,15))-(-out.obs.pstress{1}((1+i)+npx*j,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{1}((1+i)+npx*j,15))+(-out.obs.pstress{1}((1+i)+npx*j,7)))/2))/1000000;
    S3XYn(1+i,1+j) = (-out.obs.pstress{1}((1+i)+npx*j,7))/1000000;
    STXYn(1+i,1+j) = FAULT_GRIDn((1+i)+npx*j,4);
    DTXYn(1+i,1+j) = FAULT_GRIDn((1+i)+npx*j,5);
    DGXYn(1+i,1+j) = FAULT_GRIDn((1+i)+npx*j,6);
    if find(Ln(:,1)==((1+i)+npx*j))
        ScXYn(1+i,1+j) = NaN;
        S3XYn(1+i,1+j) = NaN;
    end
end
end

((Scmeann+nsd*SigScn)/1000000 - (Scmeann-nsd*SigScn)/1000000)/100

% Sc contour intervals
Vm = [(Scmeann-nsd*SigScn)/1000000:((Scmeann+nsd*SigScn)/1000000 - (Scmeann-nsd*SigScn)/1000000)/100:(Scmeann+nsd*SigScn)/1000000];
figure
contourf(Xn,Yn,ScXYn,Vm)
hold on
contour(Xn,Yn,ScXYn,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sc(MPa)','Fontsize',14);
caxis([(Scmeann-nsd*SigScn)/1000000 (Scmeann+nsd*SigScn)/1000000]);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('north AOI')

% S3 contour intervals
Vm = [(S3meann-nsd*SigS3n)/1000000:((S3meann+nsd*SigS3n)/1000000 - (S3meann-nsd*SigS3n)/1000000)/100:(S3meann+nsd*SigS3n)/1000000];
figure
contourf(Xn,Yn,S3XYn,Vm)
hold on
contour(Xn,Yn,S3XYn,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sigma3(MPa)','Fontsize',14);
caxis([(S3meann-nsd*SigS3n)/1000000 (S3meann+nsd*SigS3n)/1000000]);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('north AOI')

% slip tendency
figure
contourf(Xn,Yn,STXYn)
hold on
contour(Xn,Yn,STXYn)
c=colorbar;
set(get(c,'ylabel'),'String','Slip Tendency','Fontsize',14);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('north AOI')

% dilation tendency
figure
contourf(Xn,Yn,DTXYn)
hold on
contour(Xn,Yn,DTXYn)
c=colorbar;
set(get(c,'ylabel'),'String','Dilation Tendency','Fontsize',14);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('north AOI')

% displacement gradient
figure
clear Vm
Vm = [0:max(max(DGXYn))/100:max(max(DGXYn))];
contourf(Xn,Yn,DGXYn,Vm)
hold on
contour(Xn,Yn,DGXYn,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Slip Gradient(m/m)','Fontsize',14);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('north AOI')

% south
figure
plot([1:numel(Scs)],Scs,'k-');ylabel('Sc')
hold on
plot([0 numel(Scs)],[Scmeans+nsd*SigScs Scmeans+nsd*SigScs])
plot([0 numel(Scs)],[Scmeans-nsd*SigScs Scmeans-nsd*SigScs])
figure
plot([1:numel(S3s)],S3s,'k-');ylabel('S3')
hold on
plot([0 numel(S3s)],[S3means+nsd*SigS3s S3means+nsd*SigS3s])
plot([0 numel(S3s)],[S3means-nsd*SigS3s S3means-nsd*SigS3s])
title('south AOI')

figure
plot([1:numel(Stress.MaximumCoulombShearStress_south(:,4))],Stress.MaximumCoulombShearStress_south(:,4),'k.');ylabel('Sc favorability')
figure
plot([1:numel(Stress.Sigma3_south(:,4))],Stress.Sigma3_south(:,4),'k.');ylabel('S3 favorability')


% grid data for contour plots
npx  = 47;
npy  = 50;
for j=0:(npy-1)
for i=0:(npx-1)
    Xs(1+i,1+j) = out.obs.pstress{2}((1+i)+npx*j,1);
    Ys(1+i,1+j) = out.obs.pstress{2}((1+i)+npx*j,2);
    ScXYs(1+i,1+j) = ((((-out.obs.pstress{2}((1+i)+npx*j,15))-(-out.obs.pstress{2}((1+i)+npx*j,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{2}((1+i)+npx*j,15))+(-out.obs.pstress{2}((1+i)+npx*j,7)))/2))/1000000;
    S3XYs(1+i,1+j) = (-out.obs.pstress{2}((1+i)+npx*j,7))/1000000;
    STXYs(1+i,1+j) = FAULT_GRIDs((1+i)+npx*j,4);
    DTXYs(1+i,1+j) = FAULT_GRIDs((1+i)+npx*j,5);
    DGXYs(1+i,1+j) = FAULT_GRIDs((1+i)+npx*j,6);
    if find(Ls(:,1)==((1+i)+npx*j))
        ScXYs(1+i,1+j) = NaN;
        S3XYs(1+i,1+j) = NaN;
    end
end
end

% Sc contour intervals
Vm = [(Scmeans-nsd*SigScs)/1000000:((Scmeans+nsd*SigScs)/1000000 - (Scmeans-nsd*SigScs)/1000000)/100:(Scmeans+nsd*SigScs)/1000000];
figure
contourf(Xs,Ys,ScXYs,Vm)
hold on
contour(Xs,Ys,ScXYs,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sc(MPa)','Fontsize',14);
caxis([(Scmeans-nsd*SigScs)/1000000 (Scmeans+nsd*SigScs)/1000000]);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('south AOI')

% S3 contour intervals
Vm = [(S3means-nsd*SigS3s)/1000000:((S3means+nsd*SigS3s)/1000000 - (S3means-nsd*SigS3s)/1000000)/100:(S3means+nsd*SigS3s)/1000000];
figure
contourf(Xs,Ys,S3XYs,Vm)
hold on
contour(Xs,Ys,S3XYs,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sigma3(MPa)','Fontsize',14);
caxis([(S3means-nsd*SigS3s)/1000000 (S3means+nsd*SigS3s)/1000000]);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('south AOI')

% slip tendency
figure
contourf(Xs,Ys,STXYs)
hold on
contour(Xs,Ys,STXYs)
c=colorbar;
set(get(c,'ylabel'),'String','Slip Tendency','Fontsize',14);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('south AOI')

% dilation tendency
figure
contourf(Xs,Ys,DTXYs)
hold on
contour(Xs,Ys,DTXYs)
c=colorbar;
set(get(c,'ylabel'),'String','Dilation Tendency','Fontsize',14);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('south AOI')

% displacement gradient
figure
clear Vm
Vm = [0:max(max(DGXYs))/100:max(max(DGXYs))];
contourf(Xs,Ys,DGXYs,Vm)
hold on
contour(Xs,Ys,DGXYs,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Slip Gradient(m/m)','Fontsize',14);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
title('south AOI')