function [tn,nvec,tc,ax] = PlotTriNormals(tri,xyz,convention,FIG)

% AUTHOR:           Nick Davatzes
% DATE CREATED:     2009/21/03
% DATE MODIFIED:    2015/02/26
% VERSION:          v0.2
%
% PURPOSE:
%   Application of non-zero traction and burgers vector boundary conditions
%   to elements in p3d are defined in elocal relative to the normal vector
%   to the element. Thus the appropriate sign of normal and strike 
%   components must be chosen as a function of the normal direction. This
%   mfile:
%       1. Plots the mesh and normals to elements at the element centroids
%       2. Corrects normals to a mesh into a uniform orientation
%          - for faults this can correspond to up (positive z)
%          - for models of wells symmetric about the z-axis this can
%          correspond to outward or inward facing normals
%
% 
% NOTES: 
%   - the direction of the normal vector defining the elocal reference 
%   frame in Poly3d is defined by the order vetices identified by their 
%   index (indicating the polarity of the indices in each row of tri 
%   (an mx3 matrix defining which points form the triangular elements used 
%   in Poly3D); 
%   --> re-ordering these indices (consisting of exchanging the second and
%   thrid columns) changes the polarity, and thus the direction of the
%   normal vector
%   - Script is largely based on original scripts by J. Ole Kaven: 
%   see: tripol, tricentroid
%
% 
% INPUT:
%   xyz     mx3 array of x,y,z componetns of vertices defining mesh
%   tri     nx3 array consisting integers correspondint to row numbers of 
%           vertices from xyz that define trangular elements
%           note on .ts files: 
%               - the topology can be derived from Delaunay2_5D or by
%                 reading in .ts files using ReadTS2 and removing the first
%                 column of dat.xyz, which contains integers indexing the
%                 vertices for use in the dat.tri structured array
%               - if the min(min(dat.tri)) is 0 or if the first entry in
%                 the indices defining the vertices in the original dat.xyz
%                 is 0, then you must add 1 to the dat.tri array (since 
%                 matlab uses row numbers in its version of tri, which 
%                 always start at 1)
%   convention  Determines what convention to make all element normals match
%               - 'z+'  normal vectors point in positive z
%               - 'z-'  normal vectors point in negative z
%               - 'r+'  normal vectors in positive r direction (cylindrical 
%                       coordinates --> pointing out from the z axis)
%               - 'r-'  normal vectors point towards z-axis
%               - 'o+'  normal vectors point away from origin
%               - 'o-'  normal vectors point toward the origin
%               NOTE: if CONV is not specified, a plot of the mesh and
%               normals will be generated, but the polarity of tri will not
%               be altered.
% 
%
% OUTPUT:
%   Two plots (as subplots) of the mesh and normals located at the element
%           centroids
%   nvec    mx3 vector components of normals (each row is a normal to an
%           element, where m = length(tri)
%   tn      nxy updated tri ("connectivity") array
%   tc      mx3 x,y,z coordinates of the centroid of each triangle
%

%% 0. I/O check:
    if nargin < 2
       error('Wrong # of input')
    end
%     if nargout ~= 2
%        error('Output is a single array, wrong designation!')
%     end
    if size(tri,2)~=3
       error('Triangle element matrix should be mx3!')
    end
    if size(xyz,2)~=3
       error('Vertices should be nx3!')
    end
    
    if nargin <3
        convention = 'no'
    end
    if exist('FIG')==0
        FIG = 'N';
        ax  = [];
    end
    
%% 1. Calculate and plot normal vectors at element centroid
% 1a. Determine normal vectors
    % Construct vectors 
    v = [xyz(tri(:,3),1)-xyz(tri(:,1),1), xyz(tri(:,3),2)-xyz(tri(:,1),2), xyz(tri(:,3),3)-xyz(tri(:,1),3)];
    w = [xyz(tri(:,2),1)-xyz(tri(:,1),1), xyz(tri(:,2),2)-xyz(tri(:,1),2), xyz(tri(:,2),3)-xyz(tri(:,1),3)];
    % Calculate cross product
    normvec = [v(:,2).*w(:,3)-v(:,3).*w(:,2), ...
       -(v(:,1).*w(:,3)-v(:,3).*w(:,1)), ...
       v(:,1).*w(:,2)-v(:,2).*w(:,1)];
    % Normalize
    lnvec = sqrt(sum(normvec.*normvec,2));
    nvec = normvec./repmat(lnvec,1,3);

    
% 1b Calclute centroid of triangular elements as location of normal to element
%     cx = 1/3*(v(tri(:,1),1)+v(tri(:,2),1)+v(tri(:,3),1));
%     cy = 1/3*(v(tri(:,1),2)+v(tri(:,2),2)+v(tri(:,3),2));
%     cz = 1/3*(v(tri(:,1),3)+v(tri(:,2),3)+v(tri(:,3),3));
    cx = 1/3*(xyz(tri(:,1),1)+xyz(tri(:,2),1)+xyz(tri(:,3),1));
    cy = 1/3*(xyz(tri(:,1),2)+xyz(tri(:,2),2)+xyz(tri(:,3),2));
    cz = 1/3*(xyz(tri(:,1),3)+xyz(tri(:,2),3)+xyz(tri(:,3),3));
    tc = [cx,cy,cz];
    
% 1c Plot normals at centroids
if FIG == 'Y'
    ax(1) = subplot(1,2,1);
    S = 1; % scaling of normal vectors
    box on, hold on, grid on
    xlabel('X','FontSize',16), ylabel('Y','FontSize',16), zlabel('Z','FontSize',16)
    trisurf(tri,xyz(:,1),xyz(:,2),xyz(:,3),'EdgeColor','r','FaceColor','none')
    quiver3(cx,cy,cz,nvec(:,1),nvec(:,2),nvec(:,3),S,'g','LineWidth',0.5)
    view([0,90])
    axis equal
end
    
%% 2 Identify vectors to a consistent reference frame: update mesh & normal

% Define subfunction to change tri polarity
function [tn,nvec] = tripol_sub(t,nvec)
    tn = t;
    tn(ind01,2) = t(ind01,3);
    tn(ind01,3) = t(ind01,2);
    nvec(ind01,:) = -nvec(ind01,:);        
end

% ********************************************
% Choose the convention for normal orientation

if convention == 'z+'
% 2a.1 All normals point up (positive z)
    % Identify vectors pointing down, update mesh & normal
    tn = tri;
    ind01 = find(nvec(:,3)<0);
    [tn,nvec] = tripol_sub(tri,nvec);
%     tn(ind01,2) = t(ind01,3);
%     tn(ind01,3) = t(ind01,2);
%     nvec(ind01,:) = -nvec(ind01,:);

elseif convention == 'z-'
% 2a.1 All normals point up (positive z)
    % Identify vectors pointing down, update mesh & normal
    tn = tri;
    ind01 = find(nvec(:,3)>0);
    [tn,nvec] = tripol_sub(tri,nvec);

elseif convention == 'r+'
% 2a.2a All normals point away from the z-axis (Note: assumes that the mesh 
% is arrayed about the origin, and is based on analysis of x,y components)
    % Find normals pointing towards a local origin
    
    test = zeros(length(cx),1);
    for i = 1:length(cx)
        test(i,1) = acos(dot([cx(i),cy(i)],nvec(i,1:2))/...
            (sqrt(cx(i)^2 + cy(i)^2')*sqrt(nvec(i,1:2)*nvec(i,1:2)')));
    end
    ind01 = find(test>pi/2 | test<=-pi/2);
    [tn,nvec] = tripol_sub(tri,nvec);
     
elseif convention == 'r-'
% 2a.2b All normals point towards z-axis (Note: assumes that the mesh is
% arrayed about the origin, and is based on analysis of x,y components)
    % Find normals pointing towards a local origin
    
    test = zeros(length(cx),1);
    for i = 1:length(cx)
        test(i,1) = acos(dot([cx(i),cy(i)],nvec(i,1:2))/...
            (sqrt(cx(i)^2 + cy(i)^2')*sqrt(nvec(i,1:2)*nvec(i,1:2)')));
    end
    ind01 = find(test<pi/2 | test>=-pi/2);
    [tn,nvec] = tripol_sub(tri,nvec);

elseif convention == 'o+'
% 2a.3a All normals point away from the origin (Note: assumes that the mesh 
% is arrayed about the origin, and is based on analysis of x,y components)
    % Find normals pointing towards a local origin
    test = zeros(length(cx),1);
    for i = 1:length(cx)
        % dot of the position vector to the centroid and the normal vector
        % will be positive if pointing in same direction, negative if
        % pointing in opposing directions. Position vectors always point
        % away from the origin, providing the basis for testing the
        % orientation of the normal vector.
        test(i,1) = dot([cx(i),cy(i),cz(i)],nvec(i,:));
    end
    ind01 = find(test<0);
    [tn,nvec] = tripol_sub(tri,nvec);
elseif convention == 'o-'
% 2a.3a All normals point away from the origin (Note: assumes that the mesh 
% is arrayed about the origin, and is based on analysis of x,y components)
    % Find normals pointing towards a local origin
    test = zeros(length(cx),1);
    for i = 1:length(cx)
        % dot of the position vector to the centroid and the normal vector
        % will be positive if pointing in same direction, negative if
        % pointing in opposing directions. Position vectors always point
        % away from the origin, providing the basis for testing the
        % orientation of the normal vector.
        test(i,1) = dot([cx(i),cy(i),cz(i)],nvec(i,:));
    end
    ind01 = find(test>0);
    [tn,nvec] = tripol_sub(tri,nvec);
    
    
else % null case if no convention specified
    tn = tri;
    display('no argument defining the positive normal direction was found')
end



% ********************************************
% Visualize result of update:
% 2b Plot updated normal vectors
if FIG == 'Y'
ax(2) = subplot(1,2,2);
    S = 1; % scaling of normal vectors
    box on, hold on, grid on
    xlabel('X','FontSize',16), ylabel('Y','FontSize',16), zlabel('Z','FontSize',16)
    trisurf(tri,xyz(:,1),xyz(:,2),xyz(:,3),'EdgeColor','r','FaceColor','none')
    quiver3(cx,cy,cz,nvec(:,1),nvec(:,2),nvec(:,3),S,'g','LineWidth',0.5)
    axis equal
    view([0,90])
    linkprop(ax,'CameraPosition')
end
end


