function out = p3doutputreader_v05(filename)
% 
% Author: J. Ole Kaven
% Created:
% Version: 02
%
% Modified: 2009/12/14 N. Davatzes
% Version: 03
%
% Version: 04: ?
%
% Version: 05: 2015/3/15 M. Swyer
% 
% Modification now interprets and outputs observation points in addition to
% element data
% 
% Modification now reads multiple observation points/lines/grids/blocks. Also reads 'strains'
% instead of 'displacements' on observation grid. Comment line 183 and 
% un-comment line 182 to read 'displacements'.
% 
% 
% 
% 
% 
% 
% 
% 
% 
% New function to read variable number of faults from outputs from old
% poly3d!
%
% Generates structured array including
%   FAULT OBJECTS
%   out.fault.xyz   = array of vertex locations
% 
%   out.fault.vert  = array of vertices
%                     [VERTNUM X1 X2 X3]
% 
%   out.fault.disp  = cell array contain displacements for each fault obj
%                     [ELT X1C X2C X3C B1 U1(+) U1(-) B2 U2(+) U2(-) B3 U3(+)]
%                     ELT = element number, for use in tri
%                     X1C, X2C, X3C = element centroid coordinates
%                     B1, B2, B3 are the burgers vectos components as
%                       (strike,dip,normal)
%                     Ui(+), Ui(-) = relative displacment at centroid
%
%   out.fault.tract = cell array contain tractions for each fault obj
%                     [ELT X1C X2C X3C T1 T2 T3]
%                     T1,T2,T3 are the traction components
%                       (strike,dip,normal)
%   out.fault.name  = cell array contain displacements for each fault obj
% 
%
%   OBSERVATION GRID
%   out.obs.xyz     = array containing the obs grid pts [X1,X2,X3]
%
%   out.obs.disp    = array containing displacements at obs pts
%                     [X1 X2 X3 U1 U2 U3]
%                     U1,U2,U3 = components of the displacement vector
%                     
%   out.obs.pstress = array containing principal stress and principal
%                     stress direcitons at obs pts
%                     [X1 X2 X3 N1 N2 N3 SIG1 N1 N2 N3 SIG2 N1 N2 N3 SIG3]
%                     SIG1,2,3 = principal stress components (tensoral
%                     convention) at each observation point
%                     N1,N2,N3 = principal stress direction vectors for the
%                       following principal stress in the array)
%
%   out.obs.stress  = array contain 
%                     [X1 X2 X3 SIG11 SIG22 SIG33 SIG12 SIG23 SIG13]
%                     SIG11,22,33,12,23,13 = stress tensor component at
%                     each observation point
% 
% 
% 
% NOTES:
% 
% 
% EXAMPLE:
%   clear all
%   filename = 'Well1test.out';
% 
% 
% tic


%% Open file to write
fid1 = fopen(filename);
if(fid1==-1) 
    fprintf('Error opening input file "%s". \n',filename); 
    controls = []; return;
end
%% Scan through file line by line and distribute data into elements of the
%  structured array out
k = 0; object = 0; grid = 0; OBS = 0;
while 1                                                                     % Basically searches through each line in the opened text file fid1
    k = k+1;
    line0=(fgetl(fid1));                                                    % Gets lower case string in line (one after another)
    if ischar(line0) == 0                                                   % breaks the while loop when gets to end of file
        break
    end
    
    [word,line]=strtok(lower(line0));                                       % Returns first token in string delimited by white space
    if strcmp(word,'shear')
        [junk,line1]=strtok(line);
        [junk,line1]=strtok(line1);
        out.shearmod  = str2num(line1);
    end
%     if strcmp(word,'bulk')
%         [junk,line1]=strtok(line);
%         [junk,line1]=strtok(line1);
%         out.bulk  = str2num(line1);
%         out.pr = (3*out.bulk-2*out.shearmod)/2/(3*out.bulk+out.shearmod);
%     end
    if strcmp(word,'bulk')
        [junk,line1]=strtok(line);
        [junk,line1]=strtok(line1);
        out.bulk  = str2num(line1);
    end
    if strcmp(word,'poisson')
        [junk,line1]=strtok(line);
        [junk,line1]=strtok(line1);
        out.nu      = str2num(line1);
    end
    if strcmp(word,'young')
        [junk,line1]=strtok(line);
        [junk,line1]=strtok(line1);
        out.young   = str2num(line1);
    end
    if strcmp(word,'condition')
        [junk,line1]=strtok(line);
        [junk,line1]=strtok(line1);
        out.condnmb = str2num(line1);
    end
    % Collect data about faults
    if strcmp(word,'object:')
        object = object+1;
%         if object>10
%             break
%         end
        out.fault.name{object}=strtok(line);
        l = 0;                                                              % reset row number to zero to initialize data recording for new object
        OBS = 0;                                                            % sets up condition for recording fault vs obs data
    end
    if OBS == 0;
        if strcmp(strtok(word),'displacements:')
            pointer = 1;
        elseif strcmp(strtok(word),'stresses')
            pointer = 2; l=0;                                               % reset row number to start new array out.fault.stress (follows displacement for given fault object)
        end
        if numel(str2num(word))>0
            l = l+1;                                                        % assigns the row number for concatinationo of data at line number line0
            num01=str2num(line);
            if numel(num01)==4
                out.fault.vert{object}(l,:) = num01';
            else
                if pointer ==1
                    out.fault.disp{object}(l,:) = sscanf(line0, '%e%e%e%e%e%e%e%e%e%e%e%e%e%c*')'; 
                end
                if pointer ==2
                    out.fault.tract{object}(l,:) = sscanf(line0, '%e%e%e%e%e%e%e%c*')'; 
                end
            end
        end   
    end

%     % To end loop before observation grid...
%     if strcmp(strtok(line),'observation')
%         
%         break
%         
%     end
    
    % Collect data from observation grid (single observation grid only)
    if strcmp(strtok(line),'observation')
        grid = grid+1;
        out.obs.name{grid}=strtok(line);
        OBS = 1;
        lo = 0;
        pointer = 0;
    end
    if OBS == 1;
%           if strcmp(strtok(word),'displacements:')
       if strcmp(strtok(word),'strains:')
            pointer = 1;
%         elseif strcmp(strtok(word),'stresses:')
%             pointer = 2, lo=0
%         elseif strcmp(strtok(word),'principal')
%             pointer = 3, lo=0
        elseif strcmp(strtok(word),'stresses:')
            pointer = 2; lo=0;
        end
        if strcmp(strtok(word),'principal')
            pointer = 3; lo=0;
        end
        if numel(str2num(word))>0
            lo = lo+1;
            if pointer ==1
                out.obs.strain{grid}(lo,:) = sscanf(line0, '%e%e%e%e%e%e')'; 
            elseif pointer ==3
                out.obs.pstress{grid}(lo,:) = sscanf(line0, '%e%e%e%e%e%e%e%e%e%e%e%e%e%e%e')'; 
            elseif pointer ==2
                out.obs.stress{grid}(lo,:) = sscanf(line0, '%e%e%e%e%e%e%e%e%e')'; 
            end
        end
    end
end %while

out.fault.xyz=[];
for i = 1:object/2
    aux01 = out.fault.vert{i}(:,1);
    aux01(4:4:end) = [];
    out.fault.tri{i} = reshape(aux01,3,length(aux01)/3)';
    out.fault.xyz = [out.fault.xyz;unique(out.fault.vert{i},'rows')];
    out.fault.disp{i} = [];
end
out.fault.xyz(:,1) = [];
out.fault.disp(cellfun(@isempty,out.fault.disp))=[];
out.fault.tract(cellfun(@isempty,out.fault.tract))=[];
out.fault.name(cellfun(@isempty,out.fault.name))=[];


if OBS == 1
    out.obs.xyz{grid}(:,1:3) = out.obs.stress{grid}(:,1:3);
end

fclose(fid1);

% toc
