%% MSH_strainratev3.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    2017/05/17
% VERSION:          v.3
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: now using rotational component of strain tensor.
% with strain rates near block boundary removed.
%
% v3: get stress at 2km depth
% 
% INPUT: 2D strain rate tensor components near Mount Saint Helens from spline
%        tension model using rotational component with 0.3 spline tension and
%        modeled strainrates removed from block boundary.
%           1) MSH_n_n_rot_t0.3_bb.csv
%           2) MSH_mean_e_n_rot_t0.3_bb.csv
%           3) MSH_e_e_rot_t0.3_bb.csv
%
% OUTPUT: 3D stress tensor in Pa at 200m and 3km depths.
%
% NOTE: 
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m

clear all;close all

%% Import Data
e_e = importdata('MSH_e_e_rot_t0.3_bb.csv',',');
e_n = importdata('MSH_mean_e_n_rot_t0.3_bb.csv',',');
n_e = importdata('MSH_mean_e_n_rot_t0.3_bb.csv',',');
n_n = importdata('MSH_n_n_rot_t0.3_bb.csv',',');

for i=1:length(e_e.data)
    % 2D strain rate tensor componenets
    e{i}(1,1) = e_e.data(i,3);
    e{i}(1,2) = e_n.data(i,3);
    e{i}(2,1) = n_e.data(i,3);
    e{i}(2,2) = n_n.data(i,3);
    lat(i,1) = e_e.data(i,2);
    lon(i,1) = e_e.data(i,1);
    [v{i},d{i}] = eig(e{i});% principal 2D strain rates
    e{i}(3,3) = -.25*(d{i}(1,1) + d{i}(2,2));% vertical strain rate using Hooke's Law and Poisson's ratio of 0.25
    [v3d{i},d3d{i}] = eig(e{i});
    % 2D strain rate tensors
    E11(i) = e{i}(1,1);
    E22(i) = e{i}(2,2);
    E33(i) = e{i}(3,3);
    E12(i) = e{i}(1,2);
end
[x,y,utmzone] = deg2utm(lat,lon);

% Average 2D Strain Tensor
E(1,1) = mean(E11);E(2,2) = mean(E22);E(3,3) = mean(E33);E(1,2) = mean(E12);E(2,1) = mean(E12);
[V,D] = eig(E);% principal 2D strain rates

% MSH model boundary
latB = [46.401895 46.393716 46.056755 46.064884 46.401895];
lonB = [-122.000407 -122.528755 -122.516303 -121.991196 -122.000407];
[xB,yB,utmzone] = deg2utm(latB,lonB);
% Block Boundary
Port_YFTB = [-122.2 46.7;-122.25 46.4;-121.75 45.25];
[Port_YFTB(:,1),Port_YFTB(:,2),utmzone] = deg2utm(Port_YFTB(:,2),Port_YFTB(:,1));

%% plot 2D and 3D strain rates
% 2D strainrates
figure
h3 = plot(xB,yB,'k-');
hold on
h4 = plot(Port_YFTB(:,1),Port_YFTB(:,2),'r-','linewidth',4);
for i=1:numel(x)
    h1=quiver(x(i),y(i),v{i}(1,1),v{i}(2,1),1000,'b');
    quiver(x(i),y(i),-v{i}(1,1),-v{i}(2,1),1000,'b')
    h2=quiver(x(i),y(i),v{i}(1,2),v{i}(2,2),1000,'r');
    quiver(x(i),y(i),-v{i}(1,2),-v{i}(2,2),1000,'r')
end
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
legend([h1 h2 h3 h4],'\epsilon_H','\epsilon_h','Model Boundary','Block Boundary');
title('Mount Saint Helens Infinitesimal Strain Rate Tensors','fontsize',12)
xlim([5.2e5 6e5])
ylim([5.085e6 5.155e6])

% azimuth of maximum compression
S1az = round((180/pi)*atan(V(1,1)/V(2,1)))
S1az = (180/pi)*atan(V(1,1)/V(2,1));

% 2D strainrate
figure
h1=quiver3(V(1,1)*abs(D(1,1)),V(2,1)*abs(D(1,1)),V(3,1)*abs(D(1,1)),-V(1,1),-V(2,1),-V(3,1),abs(D(1,1)),'b');
hold on
quiver3(-V(1,1)*abs(D(1,1)),-V(2,1)*abs(D(1,1)),-V(3,1)*abs(D(1,1)),V(1,1),V(2,1),V(3,1),abs(D(1,1)),'b')
text(.7e-3,1.15e-3,strcat('S1 az=',num2str(round(S1az))))
h2=quiver3(0,0,0,-V(1,2),-V(2,2),-V(3,2),abs(D(2,2)),'g');
quiver3(0,0,0,V(1,2),V(2,2),V(3,2),abs(D(2,2)),'g')
h3=quiver3(0,0,0,-V(1,3),-V(2,3),-V(3,3),abs(D(3,3)),'r');
quiver3(0,0,0,V(1,3),V(2,3),V(3,3),abs(D(3,3)),'r')
axis equal
xlim([-1.5e-3 1.5e-3]);ylim([-1.5e-3 1.5e-3]);zlim([-1.5e-3 1.5e-3])
xlabel('X','fontsize',12);ylabel('Y','fontsize',12);zlabel('Z','fontsize',12);
view(0,90)
legend([h1 h3],strcat('\epsilon_H =',' ',num2str(D(1,1))),strcat('\epsilon_h =',' ',num2str(D(3,3))))
title('Mount Saint Helens  2D Strain Tensor','fontsize',12)

% 3D strainrate
figure
h1=quiver3(V(1,1)*abs(D(1,1)),V(2,1)*abs(D(1,1)),V(3,1)*abs(D(1,1)),-V(1,1),-V(2,1),-V(3,1),abs(D(1,1)),'b','linewidth',2);
hold on
quiver3(-V(1,1)*abs(D(1,1)),-V(2,1)*abs(D(1,1)),-V(3,1)*abs(D(1,1)),V(1,1),V(2,1),V(3,1),abs(D(1,1)),'b','linewidth',2)
h2=quiver3(0,0,0,-V(1,2),-V(2,2),-V(3,2),abs(D(2,2)),'color',[0 .5 0],'linewidth',2);
quiver3(0,0,0,V(1,2),V(2,2),V(3,2),abs(D(2,2)),'color',[0 .5 0],'linewidth',2)
h3=quiver3(0,0,0,-V(1,3),-V(2,3),-V(3,3),abs(D(3,3)),'r','linewidth',2);
quiver3(0,0,0,V(1,3),V(2,3),V(3,3),abs(D(3,3)),'r','linewidth',2)
axis equal
xlim([-1.5e-3 1.5e-3]);ylim([-1.5e-3 1.5e-3]);zlim([-1.5e-3 1.5e-3])
xlabel('X','fontsize',12);ylabel('Y','fontsize',12);zlabel('Z','fontsize',12);
legend([h1 h2 h3],strcat('\epsilon_H =',' ',num2str(D(1,1))),strcat('\epsilon_V =',' ',num2str(D(2,2))),strcat('\epsilon_h =',' ',num2str(D(3,3))))
title('Mount Saint Helens  3D Strain Tensor','fontsize',12)

%% calculate 3D remote stress
E = 3e10;% Youngs Modulus in Pa
S = D*E;
% depth to water table(meters)
dw = 20;
% 3km
de = 3;% depth km
Sv3km = 20*.1 + 27*(de-.1);
Pp3km = (9.733*(de-(dw/1000)))*1000000;
[V3km,D3km] = eig(S);
Sigma3km = D3km - (Sv3km*1000000+S(2,2));
Sigma3km(3,1:2) = 0;Sigma3km(1:2,3) = 0;
Sigma3km(2,1) = 0;Sigma3km(1,2) = 0;
Sv = Sigma3km(2,2);Sh = Sigma3km(3,3);
Sigma3km(2,2) = Sh;Sigma3km(3,3) = Sv;
A = [cos((90-S1az)*(pi/180)) -sin((90-S1az)*(pi/180)) 0;sin((90-S1az)*(pi/180)) cos((90-S1az)*(pi/180)) 0;0 0 1];
Sigma3km = A*Sigma3km*A';
[V3km,D3km] = eig(Sigma3km);

% Find 200m stress by factoring the vertical stress gradient
de = .2;% depth km
Sv200 = 20*.1 + 27*(de-.1);
Sigma200 = D3km*(Sv200/Sv3km);
SH = Sigma200(2,2);Sh = Sigma200(1,1);
Sigma200(2,2) = Sh;Sigma200(1,1) = SH;
Sigma200 = A*Sigma200*A';
[V200,D200] = eig(Sigma200);

% Find 2km stress by factoring the vertical stress gradient
Sigma2km = Sigma3km*(2/3);

%% plot stress tensor gradients
figure
h1 = plot([0 -D3km(2,2)/1e6],[0 3000],'b-');
hold on
h2 = plot([0 -D3km(3,3)/1e6],[0 3000],'-','color',[0 .5 0]);
h3 = plot([0 -D3km(1,1)/1e6],[0 3000],'r-');
plot([0 140],[200 200],'k--')
xlabel('Stress(MPa)')
ylabel('Depth(m)')
set(gca,'YDir','reverse');
legend([h1 h2 h3],'SHmax','Sv','Shmin')
title('Mount Saint Helens Stress','fontsize',12)
grid on